import base64
import datetime
from urllib.parse import urlparse

import pytest
from django.contrib.auth.models import Group
from django.urls import reverse
from webtest import Upload

from lingo.agendas.models import Agenda
from lingo.invoicing.models import (
    DEFAULT_PAYMENT_TYPES,
    AppearanceSettings,
    Campaign,
    Counter,
    DraftJournalLine,
    InjectedLine,
    JournalLine,
    Payer,
    Payment,
    PaymentType,
    Pool,
    Regie,
)
from tests.utils import login

pytestmark = pytest.mark.django_db


def test_manager_invoicing_regie_list_title(app, admin_user):
    app = login(app)
    resp = app.get(reverse('lingo-manager-invoicing-regie-list'))
    h2 = resp.pyquery('div#appbar h2')
    assert h2.text() == 'Regies'


def test_manager_invoicing_regie_list_empty(app, admin_user):
    app = login(app)
    resp = app.get(reverse('lingo-manager-invoicing-regie-list'))
    msg_info = resp.pyquery('div#content div.big-msg-info')
    assert (
        msg_info.text()
        == "This site doesn't have any regie yet. Click on the \"New\" button in the top right of the page to add a first one."
    )


def test_manager_invoicing_regie_list_add_button(app, admin_user):
    app = login(app)
    resp = app.get(reverse('lingo-manager-invoicing-regie-list'))
    add_button = resp.pyquery('a[href="%s"]' % reverse('lingo-manager-invoicing-regie-add'))
    assert add_button.text() == 'New regie'


def test_manager_invoicing_regie_list_show_objetcs(app, admin_user):
    app = login(app)
    regie = Regie.objects.create(label='Foo')
    resp = app.get(reverse('lingo-manager-invoicing-regie-list'))
    href = resp.pyquery(
        'div#content div ul li a[href="%s"]'
        % reverse('lingo-manager-invoicing-regie-detail', kwargs={'pk': regie.pk})
    )
    assert href.text() == 'Foo [identifier: foo]'


def test_manager_invoicing_regie_add(app, admin_user):
    app = login(app)
    assert Regie.objects.count() == 0
    group = Group.objects.create(name='role-foo')
    resp = app.get(reverse('lingo-manager-invoicing-regie-add'))
    h2 = resp.pyquery('div#appbar h2')
    assert h2.text() == 'New regie'
    form = resp.form
    form.set('label', 'Foo')
    form.set('description', 'foo description')
    form.set('cashier_role', group.id)
    response = form.submit().follow()
    assert Regie.objects.count() == 1
    regie = Regie.objects.first()
    assert regie.label == 'Foo'
    assert regie.slug == 'foo'
    assert regie.description == 'foo description'
    assert regie.cashier_role == group
    assert regie.paymenttype_set.count() == len(DEFAULT_PAYMENT_TYPES)
    assert urlparse(response.request.url).path == reverse(
        'lingo-manager-invoicing-regie-detail', kwargs={'pk': regie.pk}
    )


def test_manager_invoicing_regie_detail(app, admin_user):
    app = login(app)
    group = Group.objects.create(name='role-foo')
    regie = Regie.objects.create(label='Foo', description='foo description', cashier_role=group)
    resp = app.get(reverse('lingo-manager-invoicing-regie-detail', kwargs={'pk': regie.pk}))
    h2 = resp.pyquery('div#appbar h2')
    assert h2.text() == 'Regie - Foo'
    campaign = Campaign.objects.create(
        regie=regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
    )
    resp = app.get(reverse('lingo-manager-invoicing-regie-detail', kwargs={'pk': regie.pk}))
    href = resp.pyquery(
        'ul li a[href="%s"]'
        % reverse('lingo-manager-invoicing-campaign-detail', kwargs={'regie_pk': regie.pk, 'pk': campaign.pk})
    )
    assert href.text() == '(01/09/2022 - 01/10/2022) [due date: 31/10/2022]'

    campaign.finalized = True
    campaign.save()
    resp = app.get(reverse('lingo-manager-invoicing-regie-detail', kwargs={'pk': regie.pk}))
    href = resp.pyquery(
        'ul li a[href="%s"]'
        % reverse('lingo-manager-invoicing-campaign-detail', kwargs={'regie_pk': regie.pk, 'pk': campaign.pk})
    )
    assert href.text() == '(01/09/2022 - 01/10/2022) [due date: 31/10/2022] validated'


def test_manager_invoicing_regie_parameters(app, admin_user):
    app = login(app)
    group = Group.objects.create(name='role-foo')
    regie = Regie.objects.create(label='Foo', description='foo description', cashier_role=group)
    resp = app.get(reverse('lingo-manager-invoicing-regie-parameters', kwargs={'pk': regie.pk}))
    h2 = resp.pyquery('div#appbar h2')
    assert h2.text() == 'Regie - Foo'
    descr = resp.pyquery('div#panel-settings p')[0]
    assert descr.text == 'foo description'
    slug = resp.pyquery('div#panel-settings ul li')[0]
    assert slug.text == 'Identifier: foo'
    slug = resp.pyquery('div#panel-settings ul li')[1]
    assert slug.text == 'Payer: '
    cashier_role = resp.pyquery('div#panel-settings ul li')[2]
    assert cashier_role.text == 'Cashier role: role-foo'
    usage = resp.pyquery('div#panel-usage div')[0]
    assert 'This Regie is not used yet.' in usage.text
    edit_button = resp.pyquery(
        'a[href="%s"]' % reverse('lingo-manager-invoicing-regie-edit', kwargs={'pk': regie.pk})
    )
    assert edit_button.text() == 'Edit'
    delete_button = resp.pyquery(
        'a[href="%s"]' % reverse('lingo-manager-invoicing-regie-delete', kwargs={'pk': regie.pk})
    )
    assert delete_button.text() == 'Delete'

    agenda1 = Agenda.objects.create(label='Foo Bar', regie=regie)
    agenda2 = Agenda.objects.create(label='Foo Bar 2', regie=regie)
    agenda3 = Agenda.objects.create(label='Foo Bar 3')
    resp = app.get(reverse('lingo-manager-invoicing-regie-parameters', kwargs={'pk': regie.pk}))
    assert '/manage/pricing/agenda/%s/' % agenda1.pk in resp
    assert '/manage/pricing/agenda/%s/' % agenda2.pk in resp
    assert '/manage/pricing/agenda/%s/' % agenda3.pk not in resp


def test_manager_invoicing_regie_edit(app, admin_user):
    app = login(app)
    group_foo = Group.objects.create(name='role-foo')
    group_bar = Group.objects.create(name='role-bar')
    regie = Regie.objects.create(label='Foo', description='foo description', cashier_role=group_foo)
    resp = app.get(reverse('lingo-manager-invoicing-regie-edit', kwargs={'pk': regie.pk}))
    h2 = resp.pyquery('div#appbar h2')
    assert h2.text() == 'Edit regie - Foo'
    form = resp.form
    form.set('label', 'Foo bar')
    form.set('slug', 'foo-bar')
    form.set('description', 'foo new description')
    form.set('cashier_role', group_bar.id)
    response = form.submit().follow()
    assert Regie.objects.count() == 1
    regie = Regie.objects.first()
    assert regie.label == 'Foo bar'
    assert regie.slug == 'foo-bar'
    assert regie.description == 'foo new description'
    assert regie.cashier_role == group_bar
    assert urlparse(response.request.url).path == reverse(
        'lingo-manager-invoicing-regie-parameters', kwargs={'pk': regie.pk}
    )

    Regie.objects.create(label='Foo', description='foo description', cashier_role=group_foo)
    resp = app.get(reverse('lingo-manager-invoicing-regie-edit', kwargs={'pk': regie.pk}))
    form = resp.form
    form.set('slug', 'foo')
    response = form.submit()
    assert response.context['form'].errors['slug'] == ['Another regie exists with the same identifier.']

    payer = Payer.objects.create(label='Foo Bar')
    resp = app.get(reverse('lingo-manager-invoicing-regie-edit', kwargs={'pk': regie.pk}))
    form = resp.form
    form.set('payer', payer.pk)
    resp = resp.form.submit().follow()
    regie.refresh_from_db()
    assert regie.payer == payer
    assert 'Payer: <a href="/manage/invoicing/payer/%s/">Foo Bar</a>' % payer.pk in resp


def test_manager_invoicing_regie_counters_edit(app, admin_user):
    app = login(app)
    regie = Regie.objects.create(label='Foo', description='foo description')
    resp = app.get(reverse('lingo-manager-invoicing-regie-counters-edit', kwargs={'pk': regie.pk}))
    h2 = resp.pyquery('div#appbar h2')
    assert h2.text() == 'Edit counters'
    form = resp.form
    form.set('counter_name', '{yyyy}')
    form.set('invoice_number_format', 'Ffoobar-{yyyy}-{number:08d}')
    form.set('payment_number_format', 'Rfoobar-{yyyy}-{number:08d}')
    form.set('docket_number_format', 'Bfoobar-{yyyy}-{number:08d}')
    form.set('credit_number_format', 'Afoobar-{yyyy}-{number:08d}')
    form.set('refund_number_format', 'Vfoobar-{yyyy}-{number:08d}')
    resp = form.submit()
    regie.refresh_from_db()
    assert regie.counter_name == '{yyyy}'
    assert regie.invoice_number_format == 'Ffoobar-{yyyy}-{number:08d}'
    assert regie.payment_number_format == 'Rfoobar-{yyyy}-{number:08d}'
    assert regie.docket_number_format == 'Bfoobar-{yyyy}-{number:08d}'
    assert regie.credit_number_format == 'Afoobar-{yyyy}-{number:08d}'
    assert regie.refund_number_format == 'Vfoobar-{yyyy}-{number:08d}'
    assert resp.location.endswith('/manage/invoicing/regie/%s/parameters/#open:counters' % regie.pk)


def test_manager_invoicing_appearance_settings(app, admin_user, settings):
    app = login(app)
    regie = Regie.objects.create(label='Foo', description='foo description')
    assert regie.invoice_model == 'middle'
    resp = app.get(reverse('lingo-manager-invoicing-regie-list'))
    resp = resp.click('Appearance Settings')
    resp.form['logo'] = Upload(
        'test.png',
        base64.decodebytes(
            b'iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQAAAAA3bvkkAAAACklEQVQI12NoAAAAggCB3UNq9AAAAABJRU5ErkJggg=='
        ),
        'image/png',
    )
    resp.form['address'] = '<p>Foo bar<br>Streetname</p>'
    resp.form['extra_info'] = '<p>Opening hours...</p>'
    resp = resp.form.submit('submit').follow()
    appearance_settings = AppearanceSettings.singleton()
    assert appearance_settings.logo.name == 'logo/test.png'
    assert appearance_settings.address == '<p>Foo bar<br>Streetname</p>'
    assert appearance_settings.extra_info == '<p>Opening hours...</p>'
    resp = resp.click('Appearance Settings')
    assert resp.form['address'].value == '<p>Foo bar<br>Streetname</p>'

    # regie settings
    resp = app.get(reverse('lingo-manager-invoicing-regie-publishing-edit', kwargs={'pk': regie.pk}))
    resp.form.set('invoice_custom_text', '<p>custom text</p>')
    assert resp.form['invoice_model'].options == [
        ('basic', False, 'Basic'),
        ('middle', True, 'Middle'),
        ('full', False, 'Full'),
    ]
    resp.form['invoice_model'] = 'basic'
    resp.form['invoice_main_colour'] = '#9141ac'
    resp = resp.form.submit()
    regie.refresh_from_db()
    assert regie.invoice_custom_text == '<p>custom text</p>'
    assert regie.invoice_model == 'basic'
    assert regie.invoice_main_colour == '#9141ac'
    assert resp.location.endswith('/manage/invoicing/regie/%s/parameters/#open:publishing' % regie.pk)

    # check French typography fixes
    settings.LANGUAGE_CODE = 'fr-fr'
    resp = app.get(reverse('lingo-manager-invoicing-regie-publishing-edit', kwargs={'pk': regie.pk}))
    resp.form.set('invoice_custom_text', '<p>custom : text</p>')
    resp = resp.form.submit()
    regie.refresh_from_db()
    assert regie.invoice_custom_text == '<p>custom\u00a0: text</p>'


def test_manager_invoicing_regie_delete(app, admin_user):
    app = login(app)
    group = Group.objects.create(name='role-foo')
    regie = Regie.objects.create(label='Foo', description='foo description', cashier_role=group)
    assert Regie.objects.count() == 1
    resp = app.get(reverse('lingo-manager-invoicing-regie-delete', kwargs={'pk': regie.pk}))
    response = resp.form.submit().follow()
    assert Regie.objects.count() == 0
    assert urlparse(response.request.url).path == reverse('lingo-manager-invoicing-regie-list')

    # can not delete regie containing campaign
    regie.save()
    campaign = Campaign.objects.create(
        regie=regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
    )
    resp = app.get(reverse('lingo-manager-invoicing-regie-parameters', kwargs={'pk': regie.pk}))
    assert reverse('lingo-manager-invoicing-regie-delete', kwargs={'pk': regie.pk}) not in resp
    resp = app.get(reverse('lingo-manager-invoicing-regie-delete', kwargs={'pk': regie.pk}), status=404)

    campaign.delete()

    # can not delete regie containing injected line
    injected_line = InjectedLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        amount=1,
        regie=regie,
    )
    resp = app.get(reverse('lingo-manager-invoicing-regie-parameters', kwargs={'pk': regie.pk}))
    assert reverse('lingo-manager-invoicing-regie-delete', kwargs={'pk': regie.pk}) not in resp
    resp = app.get(reverse('lingo-manager-invoicing-regie-delete', kwargs={'pk': regie.pk}), status=404)

    injected_line.delete()

    # check counters are deleted
    Counter.get_count(regie=regie, name='bar', kind='invoice')
    Counter.get_count(regie=regie, name='bar', kind='payment')
    Counter.get_count(regie=regie, name='bar', kind='docket')
    Counter.get_count(regie=regie, name='bar', kind='credit')
    Counter.get_count(regie=regie, name='bar', kind='refund')
    assert Counter.objects.count() == 5
    resp = app.get(reverse('lingo-manager-invoicing-regie-delete', kwargs={'pk': regie.pk}))
    response = resp.form.submit().follow()
    assert Regie.objects.count() == 0
    assert Counter.objects.count() == 0


def test_add_payment_type(app, admin_user):
    regie = Regie.objects.create(label='Regie')

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/parameters/' % regie.pk)
    resp = resp.click('New payment type')
    resp.form['label'] = 'Foo'
    assert 'slug' not in resp.context['form'].fields
    assert 'disabled' not in resp.context['form'].fields
    resp = resp.form.submit()
    payment_type = PaymentType.objects.latest('pk')
    assert resp.location.endswith('/manage/invoicing/regie/%s/parameters/#open:payment-types' % regie.pk)
    assert payment_type.label == 'Foo'
    assert payment_type.regie == regie
    assert payment_type.slug == 'foo'
    assert payment_type.disabled is False

    resp = app.get('/manage/invoicing/regie/%s/payment-type/add/' % regie.pk)
    resp.form['label'] = 'Foo'
    resp = resp.form.submit()
    assert resp.location.endswith('/manage/invoicing/regie/%s/parameters/#open:payment-types' % regie.pk)
    payment_type = PaymentType.objects.latest('pk')
    assert payment_type.label == 'Foo'
    assert payment_type.regie == regie
    assert payment_type.slug == 'foo-1'
    assert payment_type.disabled is False


def test_edit_payment_type(app, admin_user):
    regie = Regie.objects.create(label='Regie')
    payment_type = PaymentType.objects.create(label='Foo', regie=regie)
    payment_type2 = PaymentType.objects.create(label='Baz', regie=regie)
    regie2 = Regie.objects.create(label='Regie2')
    payment_type3 = PaymentType.objects.create(label='Foo bar', regie=regie2)

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/parameters/' % regie.pk)
    resp = resp.click(href='/manage/invoicing/regie/%s/payment-type/%s/edit/' % (regie.pk, payment_type.pk))
    resp.form['label'] = 'Foo bar'
    resp.form['slug'] = payment_type2.slug
    resp.form['disabled'] = True
    resp = resp.form.submit()
    assert resp.context['form'].errors['slug'] == ['Another payment type exists with the same identifier.']

    resp.form['slug'] = payment_type3.slug
    resp = resp.form.submit()
    assert resp.location.endswith('/manage/invoicing/regie/%s/parameters/#open:payment-types' % regie.pk)
    payment_type.refresh_from_db()
    assert payment_type.label == 'Foo bar'
    assert payment_type.slug == 'foo-bar'
    assert payment_type.disabled is True

    app.get('/manage/invoicing/regie/%s/payment-type/%s/edit/' % (regie2.pk, payment_type.pk), status=404)


def test_delete_payment_type(app, admin_user):
    regie = Regie.objects.create(label='Regie')
    regie2 = Regie.objects.create(label='Regie2')
    payment_type = PaymentType.objects.create(label='Foo', regie=regie)
    payment = Payment.objects.create(
        regie=regie,
        amount=1,
        payment_type=payment_type,
    )

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/parameters/' % regie.pk)
    assert '/manage/invoicing/regie/%s/payment-type/%s/delete/' % (regie.pk, payment_type.pk) not in resp
    app.get('/manage/invoicing/regie/%s/payment-type/%s/delete/' % (regie.pk, payment_type.pk), status=404)

    payment.delete()

    app.get('/manage/invoicing/regie/%s/payment-type/%s/delete/' % (regie2.pk, payment_type.pk), status=404)

    resp = app.get('/manage/invoicing/regie/%s/parameters/' % regie.pk)
    resp = resp.click(href='/manage/invoicing/regie/%s/payment-type/%s/delete/' % (regie.pk, payment_type.pk))
    resp = resp.form.submit()
    assert resp.location.endswith('/manage/invoicing/regie/%s/parameters/#open:payment-types' % regie.pk)
    assert PaymentType.objects.exists() is False


def test_non_invoiced_line_list(app, admin_user):
    regie = Regie.objects.create(label='Regie')
    other_regie = Regie.objects.create(label='Other Regie')

    campaign = Campaign.objects.create(
        regie=regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
    )
    pool = Pool.objects.create(
        campaign=campaign,
        draft=True,
    )
    campaign2 = Campaign.objects.create(
        regie=regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
    )
    pool2 = Pool.objects.create(
        campaign=campaign2,
        draft=False,
    )

    other_campaign = Campaign.objects.create(
        regie=other_regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
    )
    other_pool = Pool.objects.create(
        campaign=other_campaign,
        draft=True,
    )
    other_campaign2 = Campaign.objects.create(
        regie=other_regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
    )
    other_pool2 = Pool.objects.create(
        campaign=other_campaign2,
        draft=False,
    )

    # not invoiced
    InjectedLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        slug='event-2022-09-01',
        label='Event 2022-09-01',
        amount=3,
        user_external_id='user:1',
        payer_external_id='payer:1',
        regie=regie,
    )
    # not invoiced but in another regie
    InjectedLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        slug='other-event-2022-09-01',
        label='Other Event 2022-09-01',
        amount=3,
        user_external_id='user:1',
        payer_external_id='payer:1',
        regie=other_regie,
    )
    # not invoiced, but linked in a DraftJournalLine
    injected_line2 = InjectedLine.objects.create(
        event_date=datetime.date(2022, 9, 2),
        slug='event-2022-09-02',
        label='Event 2022-09-02',
        amount=3,
        user_external_id='user:1',
        payer_external_id='payer:1',
        regie=regie,
    )
    DraftJournalLine.objects.create(
        event_date=datetime.date(2022, 9, 2),
        slug='event-2022-09-02',
        label='Event 2022-09-02',
        amount=3,
        pool=pool,
        from_injected_line=injected_line2,
    )
    # not invoiced, but linked in a DraftJournalLine, but in another regie
    other_injected_line2 = InjectedLine.objects.create(
        event_date=datetime.date(2022, 9, 2),
        slug='other-event-2022-09-02',
        label='Other Event 2022-09-02',
        amount=3,
        user_external_id='user:1',
        payer_external_id='payer:1',
        regie=other_regie,
    )
    DraftJournalLine.objects.create(
        event_date=datetime.date(2022, 9, 2),
        slug='other-event-2022-09-02',
        label='Other Event 2022-09-02',
        amount=3,
        pool=other_pool,
        from_injected_line=other_injected_line2,
    )
    # invoiced, as linked in a non draft pool
    injected_line3 = InjectedLine.objects.create(
        event_date=datetime.date(2022, 9, 3),
        slug='event-2022-09-03',
        label='Event 2022-09-03',
        amount=3,
        user_external_id='user:1',
        payer_external_id='payer:1',
        regie=regie,
    )
    JournalLine.objects.create(
        event_date=datetime.date(2022, 9, 3),
        slug='event-2022-09-03',
        label='Event 2022-09-03',
        amount=3,
        pool=pool2,
        from_injected_line=injected_line3,
    )

    # non fixed error
    JournalLine.objects.create(
        event_date=datetime.date(2022, 9, 4),
        slug='event-2022-09-04',
        label='Event 2022-09-04',
        amount=0,
        pool=pool2,
        status='error',
    )
    # non fixed error, but in another regie
    JournalLine.objects.create(
        event_date=datetime.date(2022, 9, 4),
        slug='other-event-2022-09-04',
        label='Other Event 2022-09-04',
        amount=0,
        pool=other_pool2,
        status='error',
    )
    # fixed or ignored errors
    JournalLine.objects.create(
        event_date=datetime.date(2022, 9, 5),
        slug='event-2022-09-05',
        label='Event 2022-09-05',
        amount=0,
        pool=pool2,
        status='error',
        error_status='fixed',
    )
    JournalLine.objects.create(
        event_date=datetime.date(2022, 9, 6),
        slug='event-2022-09-06',
        label='Event 2022-09-06',
        amount=0,
        pool=pool2,
        status='error',
        error_status='ignored',
    )
    # not errors
    JournalLine.objects.create(
        event_date=datetime.date(2022, 9, 7),
        slug='event-2022-09-07',
        label='Event 2022-09-07',
        amount=0,
        pool=pool2,
        status='success',
    )
    JournalLine.objects.create(
        event_date=datetime.date(2022, 9, 8),
        slug='event-2022-09-08',
        label='Event 2022-09-08',
        amount=0,
        pool=pool2,
        status='warning',
    )

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/non-invoiced-lines/' % regie.pk)
    assert 'event-2022-09-01' in resp
    assert 'other-event-2022-09-01' not in resp
    assert 'event-2022-09-02' in resp
    assert 'other-event-2022-09-02' not in resp
    assert 'event-2022-09-03' not in resp
    assert 'event-2022-09-04' in resp
    assert 'other-event-2022-09-04' not in resp
    assert 'event-2022-09-05' not in resp
    assert 'event-2022-09-06' not in resp
    assert 'event-2022-09-07' not in resp
    assert 'event-2022-09-08' not in resp


def test_non_invoiced_line_link(app, admin_user, settings):
    settings.KNOWN_SERVICES = {}

    regie = Regie.objects.create(label='Regie')
    campaign = Campaign.objects.create(
        regie=regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
    )
    pool = Pool.objects.create(
        campaign=campaign,
        draft=False,
    )

    line = JournalLine.objects.create(
        event_date=datetime.date(2022, 9, 4),
        slug='event-2022-09-04',
        label='Event 2022-09-04',
        amount=0,
        pool=pool,
        status='error',
    )

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/non-invoiced-lines/' % regie.pk)
    assert 'see agenda' not in resp
    assert 'see event' not in resp

    line.event = {
        'agenda': 'foobar',
    }
    line.save()
    resp = app.get('/manage/invoicing/regie/%s/non-invoiced-lines/' % regie.pk)
    assert '<a href="/manage/pricing/agenda/foobar/">see agenda</a>' in resp
    assert 'see event' not in resp

    line.event['slug'] = 'bazbaz'
    line.save()
    resp = app.get('/manage/invoicing/regie/%s/non-invoiced-lines/' % regie.pk)
    assert '<a href="/manage/pricing/agenda/foobar/">see agenda</a>' in resp
    assert 'see event' not in resp

    settings.KNOWN_SERVICES['chrono'] = {'default': {'url': 'https://chrono.dev/'}}
    resp = app.get('/manage/invoicing/regie/%s/non-invoiced-lines/' % regie.pk)
    assert '<a href="/manage/pricing/agenda/foobar/">see agenda</a>' in resp
    assert '<a href="https://chrono.dev/manage/agendas/foobar/events/bazbaz/">see event</a>' in resp
