import datetime
import decimal

import pytest
from django.utils.formats import date_format
from django.utils.timezone import localtime, now
from pyquery import PyQuery

from lingo.agendas.models import Agenda
from lingo.invoicing.models import (
    AppearanceSettings,
    Credit,
    CreditAssignment,
    CreditLine,
    Invoice,
    Payment,
    PaymentType,
    Refund,
    Regie,
)
from tests.utils import login

pytestmark = pytest.mark.django_db


def test_regie_credits(app, admin_user):
    regie = Regie.objects.create(label='Foo')
    Agenda.objects.create(label='Agenda A', regie=regie)
    Agenda.objects.create(label='Agenda B', regie=regie)
    PaymentType.create_defaults(regie)
    invoice = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        regie=regie,
    )
    credit1 = Credit.objects.create(
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
    )
    credit1.set_number()
    credit1.save()
    credit2 = Credit.objects.create(
        regie=regie,
        payer_external_id='payer:2',
        payer_first_name='First2',
        payer_last_name='Name2',
        payer_address='42 rue des kangourous\n99999 Kangourou Ville',
    )
    credit2.set_number()
    credit2.save()
    credit3 = Credit.objects.create(
        regie=regie,
        payer_external_id='payer:3',
        payer_first_name='First3',
        payer_last_name='Name3',
        payer_address='43 rue des kangourous\n99999 Kangourou Ville',
    )
    credit3.set_number()
    credit3.save()

    CreditLine.objects.create(
        slug='event-a-foo-bar',
        event_date=datetime.date(2022, 9, 1),
        credit=credit1,
        quantity=1.2,
        unit_amount=1,
        label='Event A',
        description='A description',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        event_slug='agenda-a@event-a',
        agenda_slug='agenda-a',
        activity_label='Agenda A',
    )
    CreditLine.objects.create(
        slug='event-b-foo-bar',
        event_date=datetime.date(2022, 9, 2),
        credit=credit1,
        quantity=1,
        unit_amount=2,
        label='Event B',
        user_external_id='user:2',
        user_first_name='User2',
        user_last_name='Name2',
        event_slug='agenda-b@event-b',
        agenda_slug='agenda-b',
        activity_label='Agenda B',
    )
    CreditLine.objects.create(
        slug='event-a-foo-bar',
        event_date=datetime.date(2022, 9, 3),
        credit=credit1,
        quantity=1,
        unit_amount=3,
        label='Event A',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        event_slug='agenda-a@event-a',
        agenda_slug='agenda-a',
        activity_label='Agenda A',
    )
    payment1 = Payment.objects.create(
        regie=regie,
        amount=1,
        payment_type=PaymentType.objects.get(regie=regie, slug='credit'),
    )
    payment1.set_number()
    payment1.save()
    credit_assignment1 = CreditAssignment.objects.create(
        invoice=invoice,
        payment=payment1,
        credit=credit1,
        amount=1,
    )
    refund = Refund.objects.create(
        regie=regie,
        amount=5.2,
    )
    refund.set_number()
    refund.save()
    credit_assignment2 = CreditAssignment.objects.create(
        refund=refund,
        credit=credit1,
        amount=5.2,
    )
    credit1.refresh_from_db()
    assert credit1.remaining_amount == 0
    assert credit1.assigned_amount == decimal.Decimal('6.2')

    CreditLine.objects.create(
        slug='agenda-a@event-aa',
        event_date=datetime.date(2022, 9, 1),
        credit=credit2,
        quantity=1,
        unit_amount=1,
        label='Event AA',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        event_slug='agenda-a@event-aa',
        agenda_slug='agenda-a',
        activity_label='Agenda A',
    )
    CreditAssignment.objects.create(
        invoice=invoice,
        credit=credit2,
        amount=0.5,
    )
    credit2.refresh_from_db()
    assert credit2.remaining_amount == 0.5
    assert credit2.assigned_amount == 0.5

    CreditLine.objects.create(
        slug='injected',
        event_date=datetime.date(2022, 9, 1),
        credit=credit3,
        quantity=1,
        unit_amount=1,
        label='Event A',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    credit3.refresh_from_db()
    assert credit3.remaining_amount == 1
    assert credit3.assigned_amount == 0

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/credits/' % regie.pk)
    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % credit1.pk
    ).text() == 'Assigned Credit A%02d-%s-0000001 dated %s for First1 Name1, amount 6.20€ - download' % (
        regie.pk,
        credit1.created_at.strftime('%y-%m'),
        credit1.created_at.strftime('%d/%m/%Y'),
    )
    assert len(resp.pyquery('tr[data-invoicing-element-id="%s"] a' % credit1.pk)) == 2
    lines_url = resp.pyquery('tr[data-invoicing-element-id="%s"]' % credit1.pk).attr(
        'data-invoicing-element-lines-url'
    )
    assert lines_url == '/manage/invoicing/ajax/regie/%s/credit/%s/lines/' % (
        regie.pk,
        credit1.pk,
    )
    lines_resp = app.get(lines_url)
    assert len(lines_resp.pyquery('tr')) == 14
    assert [PyQuery(tr).text() for tr in lines_resp.pyquery('tr')] == [
        'User1 Name1',
        'Description\nAmount\nQuantity\nSubtotal',
        'Agenda A',
        'Event A\nA description\n1.00€\n1.2\n1.20€',
        'Event A\n3.00€\n1\n3.00€',
        'User2 Name2',
        'Description\nAmount\nQuantity\nSubtotal',
        'Agenda B',
        'Event B\n2.00€\n1\n2.00€',
        'Assignments',
        'Payment\nDate\nAmount',
        'R%02d-%s-0000001\n%s\n1.00€'
        % (
            regie.pk,
            payment1.created_at.strftime('%y-%m'),
            date_format(localtime(credit_assignment1.created_at), 'DATETIME_FORMAT'),
        ),
        'V%02d-%s-0000001 (Refund)\n%s\n5.20€'
        % (
            regie.pk,
            refund.created_at.strftime('%y-%m'),
            date_format(localtime(credit_assignment2.created_at), 'DATETIME_FORMAT'),
        ),
        'Assigned amount: 6.20€',
    ]
    assert [PyQuery(a).attr('href') for a in lines_resp.pyquery('tr a')] == [
        '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000001'
        % (
            regie.pk,
            regie.pk,
            payment1.created_at.strftime('%y-%m'),
        ),
        '/manage/invoicing/regie/%s/refunds/?number=V%s-%s-0000001'
        % (
            regie.pk,
            regie.pk,
            refund.created_at.strftime('%y-%m'),
        ),
    ]
    assert len(resp.pyquery('tr[data-invoicing-element-id="%s"] a' % credit2.pk)) == 2
    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % credit2.pk
    ).text() == 'Partially assigned Credit A%02d-%s-0000002 dated %s for First2 Name2, amount 1.00€ - download' % (
        regie.pk,
        credit2.created_at.strftime('%y-%m'),
        credit2.created_at.strftime('%d/%m/%Y'),
    )
    lines_url = resp.pyquery('tr[data-invoicing-element-id="%s"]' % credit2.pk).attr(
        'data-invoicing-element-lines-url'
    )
    assert lines_url == '/manage/invoicing/ajax/regie/%s/credit/%s/lines/' % (
        regie.pk,
        credit2.pk,
    )
    lines_resp = app.get(lines_url)
    assert len(lines_resp.pyquery('tr')) == 9
    assert [PyQuery(tr).text() for tr in lines_resp.pyquery('tr')] == [
        'User1 Name1',
        'Description\nAmount\nQuantity\nSubtotal',
        'Agenda A',
        'Event AA\n1.00€\n1\n1.00€',
        'Assignments',
        'Payment\nDate\nAmount',
        'Pending...\n0.50€',
        'Assigned amount: 0.50€',
        'Remaining amount to assign: 0.50€',
    ]
    assert [PyQuery(a).attr('href') for a in lines_resp.pyquery('tr a')] == []

    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % credit3.pk
    ).text() == 'Credit A%02d-%s-0000003 dated %s for First3 Name3, amount 1.00€ - download' % (
        regie.pk,
        credit3.created_at.strftime('%y-%m'),
        credit3.created_at.strftime('%d/%m/%Y'),
    )
    assert len(resp.pyquery('tr[data-invoicing-element-id="%s"] a' % credit3.pk)) == 2
    lines_url = resp.pyquery('tr[data-invoicing-element-id="%s"]' % credit3.pk).attr(
        'data-invoicing-element-lines-url'
    )
    assert lines_url == '/manage/invoicing/ajax/regie/%s/credit/%s/lines/' % (
        regie.pk,
        credit3.pk,
    )
    lines_resp = app.get(lines_url)
    assert len(lines_resp.pyquery('tr')) == 7
    assert [PyQuery(tr).text() for tr in lines_resp.pyquery('tr')] == [
        'User1 Name1',
        'Description\nAmount\nQuantity\nSubtotal',
        'Event A\n1.00€\n1\n1.00€',
        'Assignments',
        'Payment\nDate\nAmount',
        'No assignments for this credit',
        'Remaining amount to assign: 1.00€',
    ]

    # test filters
    today = now().date()
    tomorrow = today + datetime.timedelta(days=1)
    yesterday = today - datetime.timedelta(days=1)
    params = [
        ({'number': credit1.formatted_number}, 1),
        ({'number': credit1.created_at.strftime('%y-%m')}, 3),
        ({'created_at_after': today.strftime('%Y-%m-%d')}, 3),
        ({'created_at_after': tomorrow.strftime('%Y-%m-%d')}, 0),
        ({'created_at_before': yesterday.strftime('%Y-%m-%d')}, 0),
        ({'created_at_before': today.strftime('%Y-%m-%d')}, 3),
        ({'payment_number': payment1.formatted_number}, 1),
        ({'payment_number': payment1.created_at.strftime('%y-%m')}, 1),
        ({'payer_external_id': 'payer:1'}, 1),
        ({'payer_external_id': 'payer:2'}, 1),
        ({'payer_first_name': 'first'}, 3),
        ({'payer_first_name': 'first1'}, 1),
        ({'payer_last_name': 'name'}, 3),
        ({'payer_last_name': 'name1'}, 1),
        ({'user_external_id': 'user:1'}, 3),
        ({'user_external_id': 'user:2'}, 1),
        ({'user_first_name': 'user'}, 3),
        ({'user_first_name': 'user2'}, 1),
        ({'user_last_name': 'name'}, 3),
        ({'user_last_name': 'name1'}, 3),
        (
            {
                'total_amount_min': '1',
                'total_amount_min_lookup': 'gt',
            },
            1,
        ),
        (
            {
                'total_amount_min': '1',
                'total_amount_min_lookup': 'gte',
            },
            3,
        ),
        (
            {
                'total_amount_max': '6.2',
                'total_amount_max_lookup': 'lt',
            },
            2,
        ),
        (
            {
                'total_amount_max': '6.2',
                'total_amount_max_lookup': 'lte',
            },
            3,
        ),
        ({'assigned': 'yes'}, 1),
        ({'assigned': 'partially'}, 1),
        ({'assigned': 'no'}, 1),
        ({'agenda': 'agenda-a'}, 2),
        ({'agenda': 'agenda-b'}, 1),
        ({'event': 'agenda-a@event-a'}, 1),
        ({'event': 'agenda-a@event-aa'}, 1),
        ({'event': 'agenda-b@event-b'}, 1),
    ]
    for param, result in params:
        resp = app.get(
            '/manage/invoicing/regie/%s/credits/' % regie.pk,
            params=param,
        )
        assert len(resp.pyquery('tr.credit')) == result


def test_regie_credit_pdf(app, admin_user):
    regie = Regie.objects.create(label='Foo', invoice_main_colour='#9141ac')
    credit = Credit.objects.create(
        label='Credit from 01/09/2022',
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
    )
    credit.set_number()
    credit.save()

    CreditLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        credit=credit,
        quantity=1.2,
        unit_amount=1,
        label='Label 11',
        description='A description',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        event_slug='agenda-a@event-a',
        agenda_slug='agenda-a',
        activity_label='Agenda A',
    )
    CreditLine.objects.create(
        event_date=datetime.date(2022, 9, 2),
        credit=credit,
        quantity=1,
        unit_amount=2,
        label='Label 12',
        user_external_id='user:2',
        user_first_name='User2',
        user_last_name='Name2',
        event_slug='agenda-a@event-a',
        agenda_slug='agenda-a',
        activity_label='Agenda A',
    )
    CreditLine.objects.create(
        event_date=datetime.date(2022, 9, 3),
        credit=credit,
        quantity=1,
        unit_amount=3,
        label='Label 13',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        event_slug='agenda-b@event-b',
        agenda_slug='agenda-b',
        activity_label='Agenda B',
    )

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/credit/%s/pdf/?html' % (regie.pk, credit.pk))
    assert 'color: #9141ac;' in resp
    assert resp.pyquery('#document-label').text() == 'Credit from 01/09/2022'
    assert resp.pyquery('#regie-label').text() == 'Foo'
    assert resp.pyquery('address#to').text() == 'First1 Name1\n41 rue des kangourous\n99999 Kangourou Ville'
    assert resp.pyquery('dl#informations').text() == 'Credit number:\nA%02d-%s-0000001\nDate:\n%s' % (
        regie.pk,
        credit.created_at.strftime('%y-%m'),
        date_format(localtime(credit.created_at), 'DATE_FORMAT'),
    )
    assert [PyQuery(tr).text() for tr in resp.pyquery('table#lines tr')] == [
        'User1 Name1',
        'Services\nDetails\nUA\nQTY\nTA',
        'Label 11\nAgenda A\n\nA description\n1.00€\n1.2\n1.20€',
        'Label 13\nAgenda B\n\n3.00€\n1\n3.00€',
        'User2 Name2',
        'Services\nDetails\nUA\nQTY\nTA',
        'Label 12\nAgenda A\n\n2.00€\n1\n2.00€',
        'Total amount:\n6.20€',
    ]

    app.get('/manage/invoicing/regie/%s/credit/%s/pdf/?html' % (0, credit.pk), status=404)
    app.get('/manage/invoicing/regie/%s/credit/%s/pdf/?html' % (regie.pk, 0), status=404)
    other_regie = Regie.objects.create(label='Foo')
    app.get('/manage/invoicing/regie/%s/credit/%s/pdf/?html' % (other_regie.pk, credit.pk), status=404)

    appearance_settings = AppearanceSettings.singleton()
    appearance_settings.address = '<p>Foo bar<br>Streetname</p>'
    appearance_settings.extra_info = '<p>Opening hours...</p>'
    appearance_settings.save()
    resp = app.get('/manage/invoicing/regie/%s/credit/%s/pdf/?html' % (regie.pk, credit.pk))
    assert appearance_settings.address in resp.text
    assert appearance_settings.extra_info in resp.text


def test_regie_refunds(app, admin_user):
    regie = Regie.objects.create(label='Foo')
    Agenda.objects.create(label='Agenda A', regie=regie)
    Agenda.objects.create(label='Agenda B', regie=regie)
    PaymentType.create_defaults(regie)
    invoice = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        regie=regie,
    )
    credit1 = Credit.objects.create(
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
    )
    credit1.set_number()
    credit1.save()
    credit2 = Credit.objects.create(
        regie=regie,
        payer_external_id='payer:2',
        payer_first_name='First2',
        payer_last_name='Name2',
        payer_address='42 rue des kangourous\n99999 Kangourou Ville',
    )
    credit2.set_number()
    credit2.save()
    credit3 = Credit.objects.create(
        regie=regie,
        payer_external_id='payer:3',
        payer_first_name='First3',
        payer_last_name='Name3',
        payer_address='43 rue des kangourous\n99999 Kangourou Ville',
    )
    credit3.set_number()
    credit3.save()

    CreditLine.objects.create(
        slug='event-a-foo-bar',
        event_date=datetime.date(2022, 9, 1),
        credit=credit1,
        quantity=1.2,
        unit_amount=1,
        label='Event A',
        description='A description',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    CreditLine.objects.create(
        slug='event-b-foo-bar',
        event_date=datetime.date(2022, 9, 2),
        credit=credit1,
        quantity=1,
        unit_amount=2,
        label='Event B',
        user_external_id='user:2',
        user_first_name='User2',
        user_last_name='Name2',
    )
    CreditLine.objects.create(
        slug='event-a-foo-bar',
        event_date=datetime.date(2022, 9, 3),
        credit=credit1,
        quantity=1,
        unit_amount=3,
        label='Event A',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    refund1 = Refund.objects.create(
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        amount=6.2,
    )
    refund1.set_number()
    refund1.save()
    CreditAssignment.objects.create(
        refund=refund1,
        credit=credit1,
        amount=6.2,
    )
    credit1.refresh_from_db()
    assert credit1.remaining_amount == 0
    assert credit1.assigned_amount == decimal.Decimal('6.2')

    CreditLine.objects.create(
        slug='agenda-a@event-aa',
        event_date=datetime.date(2022, 9, 1),
        credit=credit2,
        quantity=1,
        unit_amount=1,
        label='Event AA',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    payment2 = Payment.objects.create(
        regie=regie,
        amount=0.5,
        payment_type=PaymentType.objects.get(regie=regie, slug='credit'),
    )
    payment2.set_number()
    payment2.save()
    CreditAssignment.objects.create(
        invoice=invoice,
        payment=payment2,
        credit=credit2,
        amount=0.5,
    )
    refund2 = Refund.objects.create(
        regie=regie,
        payer_external_id='payer:2',
        payer_first_name='First2',
        payer_last_name='Name2',
        payer_address='42 rue des kangourous\n99999 Kangourou Ville',
        amount=0.5,
    )
    refund2.set_number()
    refund2.save()
    CreditAssignment.objects.create(
        refund=refund2,
        credit=credit2,
        amount=0.5,
    )
    credit2.refresh_from_db()
    assert credit2.remaining_amount == 0
    assert credit2.assigned_amount == 1

    CreditLine.objects.create(
        slug='injected',
        event_date=datetime.date(2022, 9, 1),
        credit=credit3,
        quantity=1,
        unit_amount=1,
        label='Event A',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    refund3 = Refund.objects.create(
        regie=regie,
        payer_external_id='payer:3',
        payer_first_name='First3',
        payer_last_name='Name3',
        payer_address='43 rue des kangourous\n99999 Kangourou Ville',
        amount=1,
    )
    refund3.set_number()
    refund3.save()
    CreditAssignment.objects.create(
        refund=refund3,
        credit=credit3,
        amount=1,
    )
    credit3.refresh_from_db()
    assert credit3.remaining_amount == 0
    assert credit3.assigned_amount == 1

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/refunds/' % regie.pk)
    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % refund1.pk
    ).text() == 'Refund V%02d-%s-0000001 dated %s for First1 Name1, amount 6.20€' % (
        regie.pk,
        refund1.created_at.strftime('%y-%m'),
        refund1.created_at.strftime('%d/%m/%Y'),
    )
    assert [
        PyQuery(tr).text() for tr in resp.pyquery('tr[data-related-invoicing-element-id="%s"]' % refund1.pk)
    ] == [
        'Credit\nDate\nCredit amount\nRefund amount',
        'A%02d-%s-0000001\n%s\n6.20€\n6.20€'
        % (
            regie.pk,
            credit1.created_at.strftime('%y-%m'),
            credit1.created_at.strftime('%d/%m/%Y'),
        ),
    ]

    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % refund2.pk
    ).text() == 'Refund V%02d-%s-0000002 dated %s for First2 Name2, amount 0.50€' % (
        regie.pk,
        refund2.created_at.strftime('%y-%m'),
        refund2.created_at.strftime('%d/%m/%Y'),
    )
    assert [
        PyQuery(tr).text() for tr in resp.pyquery('tr[data-related-invoicing-element-id="%s"]' % refund2.pk)
    ] == [
        'Credit\nDate\nCredit amount\nRefund amount',
        'A%02d-%s-0000002\n%s\n1.00€\n0.50€'
        % (
            regie.pk,
            credit2.created_at.strftime('%y-%m'),
            credit2.created_at.strftime('%d/%m/%Y'),
        ),
    ]

    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % refund3.pk
    ).text() == 'Refund V%02d-%s-0000003 dated %s for First3 Name3, amount 1.00€' % (
        regie.pk,
        refund3.created_at.strftime('%y-%m'),
        refund3.created_at.strftime('%d/%m/%Y'),
    )
    assert [
        PyQuery(tr).text() for tr in resp.pyquery('tr[data-related-invoicing-element-id="%s"]' % refund3.pk)
    ] == [
        'Credit\nDate\nCredit amount\nRefund amount',
        'A%02d-%s-0000003\n%s\n1.00€\n1.00€'
        % (
            regie.pk,
            credit3.created_at.strftime('%y-%m'),
            credit3.created_at.strftime('%d/%m/%Y'),
        ),
    ]

    # test filters
    today = now().date()
    tomorrow = today + datetime.timedelta(days=1)
    yesterday = today - datetime.timedelta(days=1)
    params = [
        ({'number': refund1.formatted_number}, 1),
        ({'number': refund1.created_at.strftime('%y-%m')}, 3),
        ({'created_at_after': today.strftime('%Y-%m-%d')}, 3),
        ({'created_at_after': tomorrow.strftime('%Y-%m-%d')}, 0),
        ({'created_at_before': yesterday.strftime('%Y-%m-%d')}, 0),
        ({'created_at_before': today.strftime('%Y-%m-%d')}, 3),
        ({'credit_number': credit1.formatted_number}, 1),
        ({'credit_number': credit1.created_at.strftime('%y-%m')}, 3),
        ({'payer_external_id': 'payer:1'}, 1),
        ({'payer_external_id': 'payer:2'}, 1),
        ({'payer_first_name': 'first'}, 3),
        ({'payer_first_name': 'first1'}, 1),
        ({'payer_last_name': 'name'}, 3),
        ({'payer_last_name': 'name1'}, 1),
        (
            {
                'amount_min': '1',
                'amount_min_lookup': 'gt',
            },
            1,
        ),
        (
            {
                'amount_min': '1',
                'amount_min_lookup': 'gte',
            },
            2,
        ),
        (
            {
                'amount_max': '6.2',
                'amount_max_lookup': 'lt',
            },
            2,
        ),
        (
            {
                'amount_max': '6.2',
                'amount_max_lookup': 'lte',
            },
            3,
        ),
    ]
    for param, result in params:
        resp = app.get(
            '/manage/invoicing/regie/%s/refunds/' % regie.pk,
            params=param,
        )
        assert len(resp.pyquery('tr.refund')) == result
