import datetime
import decimal

import pytest
from django.utils.formats import date_format
from django.utils.timezone import localtime, now
from pyquery import PyQuery

from lingo.agendas.models import Agenda
from lingo.invoicing.models import (
    AppearanceSettings,
    Campaign,
    Invoice,
    InvoiceLine,
    InvoiceLinePayment,
    Payment,
    PaymentType,
    Pool,
    Regie,
)
from tests.utils import login

pytestmark = pytest.mark.django_db


@pytest.mark.parametrize('orphan', [True, False])
def test_regie_invoices(app, admin_user, orphan):
    regie = Regie.objects.create(label='Foo')
    Agenda.objects.create(label='Agenda A', regie=regie)
    Agenda.objects.create(label='Agenda B', regie=regie)
    PaymentType.create_defaults(regie)
    today = now().date()
    tomorrow = today + datetime.timedelta(days=1)
    yesterday = today - datetime.timedelta(days=1)
    if not orphan:
        campaign1 = Campaign.objects.create(
            regie=regie,
            date_start=datetime.date(2022, 9, 1),
            date_end=datetime.date(2022, 10, 1),
            date_publication=datetime.date(2022, 10, 1),
            date_payment_deadline=datetime.date(2022, 10, 31),
            date_due=datetime.date(2022, 10, 31),
            date_debit=datetime.date(2022, 11, 15),
            finalized=True,
        )
        pool1 = Pool.objects.create(
            campaign=campaign1,
            draft=False,
            status='completed',
        )
        campaign2 = Campaign.objects.create(
            regie=regie,
            date_start=datetime.date(2022, 9, 1),
            date_end=datetime.date(2022, 10, 1),
            date_publication=datetime.date(2022, 10, 1),
            date_payment_deadline=datetime.date(2022, 10, 31),
            date_due=datetime.date(2022, 10, 31),
            date_debit=datetime.date(2022, 11, 15),
            finalized=True,
        )
        pool2 = Pool.objects.create(
            campaign=campaign2,
            draft=False,
            status='completed',
        )
    invoice1 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        regie=regie,
        pool=pool1 if not orphan else None,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=True,
        payer_direct_debit=False,
    )
    invoice1.set_number()
    invoice1.save()
    invoice2 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
        regie=regie,
        pool=pool2 if not orphan else None,
        payer_external_id='payer:2',
        payer_first_name='First2',
        payer_last_name='Name2',
        payer_address='42 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=False,
        payer_direct_debit=True,
    )
    invoice2.set_number()
    invoice2.save()
    invoice3 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
        regie=regie,
        pool=pool2 if not orphan else None,
        payer_external_id='payer:3',
        payer_first_name='First3',
        payer_last_name='Name3',
        payer_address='43 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=False,
        payer_direct_debit=True,
    )
    invoice3.set_number()
    invoice3.save()
    invoice4 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        regie=regie,
        cancelled_at=now(),
    )
    invoice4.set_number()
    invoice4.save()

    invoice_line11 = InvoiceLine.objects.create(
        slug='event-a-foo-bar',
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice1,
        quantity=1.2,
        unit_amount=1,
        pool=pool1 if not orphan else None,
        label='Event A',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        details={
            'agenda': 'agenda-a',
            'primary_event': 'event-a',
            'status': 'presence',
            'check_type': 'foo',
            'check_type_group': 'foobar',
            'check_type_label': 'Foo!',
            'dates': ['2022-09-01', '2022-09-02', '2022-09-03'],
        },
        event_slug='agenda-a@event-a',
        agenda_slug='agenda-a',
        activity_label='Agenda A',
        description='Thu01, Fri02, Sat03',
    )
    invoice_line12 = InvoiceLine.objects.create(
        slug='event-b-foo-bar',
        event_date=datetime.date(2022, 9, 2),
        invoice=invoice1,
        quantity=1,
        unit_amount=2,
        pool=pool1 if not orphan else None,
        label='Event B',
        user_external_id='user:2',
        user_first_name='User2',
        user_last_name='Name2',
        details={
            'agenda': 'agenda-b',
            'primary_event': 'event-b',
            'status': 'presence',
            'check_type': 'foo',
            'check_type_group': 'foobar',
            'check_type_label': 'Foo!',
            'dates': ['2022-09-01', '2022-09-02', '2022-09-03'],
        },
        event_slug='agenda-b@event-b',
        agenda_slug='agenda-b',
        activity_label='Agenda B',
        description='Thu01, Fri02, Sat03',
    )
    invoice_line13 = InvoiceLine.objects.create(
        slug='event-a-foo-bar',
        event_date=datetime.date(2022, 9, 3),
        invoice=invoice1,
        quantity=1,
        unit_amount=3,
        pool=pool1 if not orphan else None,
        label='Event A',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        details={
            'agenda': 'agenda-a',
            'primary_event': 'event-a',
            'status': 'presence',
            'dates': ['2022-09-04', '2022-09-05'],
        },
        event_slug='agenda-a@event-a',
        agenda_slug='agenda-a',
        activity_label='Agenda A',
        description='Sun04, Mon05',
    )
    payment1 = Payment.objects.create(
        regie=regie,
        amount=1,
        payment_type=PaymentType.objects.get(regie=regie, slug='cash'),
    )
    payment1.set_number()
    payment1.save()
    invoice_payment1 = InvoiceLinePayment.objects.create(
        payment=payment1,
        line=invoice_line11,
        amount=1,
    )
    invoice1.refresh_from_db()
    assert invoice1.remaining_amount == decimal.Decimal('5.2')
    assert invoice1.paid_amount == 1

    invoice_line21 = InvoiceLine.objects.create(
        slug='agenda-a@event-aa',  # non recurring event
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice2,
        quantity=1,
        unit_amount=1,
        pool=pool2 if not orphan else None,
        label='Event AA',
        event_slug='agenda-a@event-aa',
        agenda_slug='agenda-a',
        activity_label='Agenda A',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    payment2 = Payment.objects.create(
        regie=regie,
        amount=0.5,
        payment_type=PaymentType.objects.get(regie=regie, slug='cash'),
    )
    payment2.set_number()
    payment2.save()
    invoice_payment2 = InvoiceLinePayment.objects.create(
        payment=payment2,
        line=invoice_line21,
        amount=0.5,
    )
    payment3 = Payment.objects.create(
        regie=regie,
        amount=0.5,
        payment_type=PaymentType.objects.get(regie=regie, slug='cesu'),
    )
    payment3.set_number()
    payment3.save()
    invoice_payment3 = InvoiceLinePayment.objects.create(
        payment=payment3,
        line=invoice_line21,
        amount=0.5,
    )
    invoice2.refresh_from_db()
    assert invoice2.remaining_amount == 0
    assert invoice2.paid_amount == 1

    invoice_line31 = InvoiceLine.objects.create(
        slug='injected',  # from injected line
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice3,
        quantity=1,
        unit_amount=1,
        pool=pool2 if not orphan else None,
        label='Event A',
        event_slug='injected',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    invoice3.refresh_from_db()
    assert invoice3.remaining_amount == 1
    assert invoice3.paid_amount == 0

    non_finalized_campaign = Campaign.objects.create(
        regie=regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
    )
    non_finalized_pool = Pool.objects.create(
        campaign=non_finalized_campaign,
        draft=False,
        status='completed',
    )
    non_finalized_invoice = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        regie=regie,
        pool=non_finalized_pool,
        payer_external_id='payer:1',
    )
    non_finalized_invoice.set_number()
    non_finalized_invoice.save()
    InvoiceLine.objects.create(
        slug='event-a-foo-bar',
        event_date=datetime.date(2022, 9, 1),
        invoice=non_finalized_invoice,
        quantity=1.2,
        unit_amount=1,
        pool=non_finalized_pool,
        label='Event A',
        event_slug='event-a-foo-bar',
        user_external_id='user:1',
        description='@overtaking@',
    )

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/invoices/' % regie.pk)
    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % invoice1.pk
    ).text() == 'Partially paid Invoice F%02d-%s-0000001 dated %s addressed to First1 Name1, amount 6.20€ - download' % (
        regie.pk,
        invoice1.created_at.strftime('%y-%m'),
        invoice1.created_at.strftime('%d/%m/%Y'),
    )
    if orphan:
        assert len(resp.pyquery('tr[data-invoicing-element-id="%s"] a' % invoice1.pk)) == 2
    else:
        assert len(resp.pyquery('tr[data-invoicing-element-id="%s"] a' % invoice1.pk)) == 3
    lines_url = resp.pyquery('tr[data-invoicing-element-id="%s"]' % invoice1.pk).attr(
        'data-invoicing-element-lines-url'
    )
    assert lines_url == '/manage/invoicing/ajax/regie/%s/invoice/%s/lines/' % (
        regie.pk,
        invoice1.pk,
    )
    lines_resp = app.get(lines_url)
    assert len(lines_resp.pyquery('tr')) == 28
    assert [PyQuery(tr).text() for tr in lines_resp.pyquery('tr')] == [
        'Demat: yes',
        'Direct debit: no',
        'Publication date: 01/10/2022',
        'Payment deadline: 31/10/2022',
        'Due date: 31/10/2022',
        'User1 Name1',
        'Description\nAmount\nQuantity\nSubtotal',
        'Agenda A',
        'Event A\nFoo! Thu01, Fri02, Sat03\n1.00€\n1.2\n1.20€',
        'Payments',
        'Payment\nDate\nType\nAmount',
        'R%02d-%s-0000001\n%s\nCash\n1.00€'
        % (
            regie.pk,
            payment1.created_at.strftime('%y-%m'),
            date_format(localtime(invoice_payment1.created_at), 'DATETIME_FORMAT'),
        ),
        'Event A\nPresence Sun04, Mon05\n3.00€\n1\n3.00€',
        'Payments',
        'Payment\nDate\nType\nAmount',
        'No payments for this line',
        'User2 Name2',
        'Description\nAmount\nQuantity\nSubtotal',
        'Agenda B',
        'Event B\nFoo! Thu01, Fri02, Sat03\n2.00€\n1\n2.00€',
        'Payments',
        'Payment\nDate\nType\nAmount',
        'No payments for this line',
        'Payments',
        'Payment\nDate\nType\nAmount',
        'R%02d-%s-0000001\n%s\nCash\n1.00€'
        % (
            regie.pk,
            payment1.created_at.strftime('%y-%m'),
            date_format(localtime(invoice_payment1.created_at), 'DATETIME_FORMAT'),
        ),
        'Paid amount: 1.00€',
        'Remaining amount: 5.20€',
    ]
    if not orphan:
        assert [PyQuery(a).attr('href') for a in lines_resp.pyquery('tr a')] == [
            '/manage/invoicing/regie/%s/campaign/%s/pool/%s/journal/?user_external_id=user:1'
            % (regie.pk, campaign1.pk, pool1.pk),
            '/manage/invoicing/regie/%s/campaign/%s/pool/%s/journal/?invoice_line=%s'
            % (regie.pk, campaign1.pk, pool1.pk, invoice_line11.pk),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000001'
            % (
                regie.pk,
                regie.pk,
                payment1.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/campaign/%s/pool/%s/journal/?invoice_line=%s'
            % (regie.pk, campaign1.pk, pool1.pk, invoice_line13.pk),
            '/manage/invoicing/regie/%s/campaign/%s/pool/%s/journal/?user_external_id=user:2'
            % (regie.pk, campaign1.pk, pool1.pk),
            '/manage/invoicing/regie/%s/campaign/%s/pool/%s/journal/?invoice_line=%s'
            % (regie.pk, campaign1.pk, pool1.pk, invoice_line12.pk),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000001'
            % (
                regie.pk,
                regie.pk,
                payment1.created_at.strftime('%y-%m'),
            ),
        ]
    else:
        assert [PyQuery(a).attr('href') for a in lines_resp.pyquery('tr a')] == [
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000001'
            % (
                regie.pk,
                regie.pk,
                payment1.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000001'
            % (
                regie.pk,
                regie.pk,
                payment1.created_at.strftime('%y-%m'),
            ),
        ]
    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % invoice2.pk
    ).text() == 'Paid Invoice F%02d-%s-0000002 dated %s addressed to First2 Name2, amount 1.00€ - download' % (
        regie.pk,
        invoice2.created_at.strftime('%y-%m'),
        invoice2.created_at.strftime('%d/%m/%Y'),
    )
    lines_url = resp.pyquery('tr[data-invoicing-element-id="%s"]' % invoice2.pk).attr(
        'data-invoicing-element-lines-url'
    )
    assert lines_url == '/manage/invoicing/ajax/regie/%s/invoice/%s/lines/' % (
        regie.pk,
        invoice2.pk,
    )
    lines_resp = app.get(lines_url)
    assert len(lines_resp.pyquery('tr')) == 20
    assert [PyQuery(tr).text() for tr in lines_resp.pyquery('tr')] == [
        'Demat: no',
        'Direct debit: no',
        'Publication date: 01/10/2022',
        'Payment deadline: 31/10/2022',
        'Due date: 31/10/2022',
        'Debit date: 15/11/2022',
        'User1 Name1',
        'Description\nAmount\nQuantity\nSubtotal',
        'Agenda A',
        'Event AA\n1.00€\n1\n1.00€',
        'Payments',
        'Payment\nDate\nType\nAmount',
        'R%02d-%s-0000002\n%s\nCash\n0.50€'
        % (
            regie.pk,
            payment2.created_at.strftime('%y-%m'),
            date_format(localtime(invoice_payment2.created_at), 'DATETIME_FORMAT'),
        ),
        'R%02d-%s-0000003\n%s\nCESU\n0.50€'
        % (
            regie.pk,
            payment3.created_at.strftime('%y-%m'),
            date_format(localtime(invoice_payment3.created_at), 'DATETIME_FORMAT'),
        ),
        'Payments',
        'Payment\nDate\nType\nAmount',
        'R%02d-%s-0000002\n%s\nCash\n0.50€'
        % (
            regie.pk,
            payment2.created_at.strftime('%y-%m'),
            date_format(localtime(invoice_payment2.created_at), 'DATETIME_FORMAT'),
        ),
        'R%02d-%s-0000003\n%s\nCESU\n0.50€'
        % (
            regie.pk,
            payment3.created_at.strftime('%y-%m'),
            date_format(localtime(invoice_payment3.created_at), 'DATETIME_FORMAT'),
        ),
        'Paid amount: 1.00€',
        'Payments certificate: download',
    ]
    if not orphan:
        assert [PyQuery(a).attr('href') for a in lines_resp.pyquery('tr a')] == [
            '/manage/invoicing/regie/%s/campaign/%s/pool/%s/journal/?user_external_id=user:1'
            % (regie.pk, campaign2.pk, pool2.pk),
            '/manage/invoicing/regie/%s/campaign/%s/pool/%s/journal/?invoice_line=%s'
            % (regie.pk, campaign2.pk, pool2.pk, invoice_line21.pk),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000002'
            % (
                regie.pk,
                regie.pk,
                payment2.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000003'
            % (
                regie.pk,
                regie.pk,
                payment3.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000002'
            % (
                regie.pk,
                regie.pk,
                payment2.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000003'
            % (
                regie.pk,
                regie.pk,
                payment3.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/invoice/%s/payments/pdf/' % (regie.pk, invoice2.pk),
        ]
    else:
        assert [PyQuery(a).attr('href') for a in lines_resp.pyquery('tr a')] == [
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000002'
            % (
                regie.pk,
                regie.pk,
                payment2.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000003'
            % (
                regie.pk,
                regie.pk,
                payment3.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000002'
            % (
                regie.pk,
                regie.pk,
                payment2.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/payments/?number=R%s-%s-0000003'
            % (
                regie.pk,
                regie.pk,
                payment3.created_at.strftime('%y-%m'),
            ),
            '/manage/invoicing/regie/%s/invoice/%s/payments/pdf/' % (regie.pk, invoice2.pk),
        ]

    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % invoice3.pk
    ).text() == 'Invoice F%02d-%s-0000003 dated %s addressed to First3 Name3, amount 1.00€ - download' % (
        regie.pk,
        invoice3.created_at.strftime('%y-%m'),
        invoice3.created_at.strftime('%d/%m/%Y'),
    )
    if orphan:
        assert len(resp.pyquery('tr[data-invoicing-element-id="%s"] a' % invoice3.pk)) == 2
    else:
        assert len(resp.pyquery('tr[data-invoicing-element-id="%s"] a' % invoice3.pk)) == 3
    lines_url = resp.pyquery('tr[data-invoicing-element-id="%s"]' % invoice3.pk).attr(
        'data-invoicing-element-lines-url'
    )
    assert lines_url == '/manage/invoicing/ajax/regie/%s/invoice/%s/lines/' % (
        regie.pk,
        invoice3.pk,
    )
    lines_resp = app.get(lines_url)
    assert len(lines_resp.pyquery('tr')) == 16
    assert [PyQuery(tr).text() for tr in lines_resp.pyquery('tr')] == [
        'Demat: no',
        'Direct debit: no',
        'Publication date: 01/10/2022',
        'Payment deadline: 31/10/2022',
        'Due date: 31/10/2022',
        'Debit date: 15/11/2022',
        'User1 Name1',
        'Description\nAmount\nQuantity\nSubtotal',
        'Event A\n1.00€\n1\n1.00€',
        'Payments',
        'Payment\nDate\nType\nAmount',
        'No payments for this line',
        'Payments',
        'Payment\nDate\nType\nAmount',
        'No payments for this invoice',
        'Remaining amount: 1.00€',
    ]
    if not orphan:
        assert [PyQuery(a).attr('href') for a in lines_resp.pyquery('tr a')] == [
            '/manage/invoicing/regie/%s/campaign/%s/pool/%s/journal/?user_external_id=user:1'
            % (regie.pk, campaign2.pk, pool2.pk),
            '/manage/invoicing/regie/%s/campaign/%s/pool/%s/journal/?invoice_line=%s'
            % (regie.pk, campaign2.pk, pool2.pk, invoice_line31.pk),
        ]

    resp = app.get('/manage/invoicing/regie/%s/invoices/?csv' % regie.pk)
    assert resp.headers['Content-Type'] == 'text/csv'
    assert resp.headers['Content-Disposition'] == 'attachment; filename="invoices.csv"'
    assert len([a for a in resp.text.split('\r\n') if a]) == 1 + 3
    assert resp.text == (
        'Number,Payer ID,Payer first name,Payer last name,Publication date,Payment deadline,Due date,Demat,Direct debit,Total due,Paid amount,Status\r\n'
        'F%02d-%s-0000003,payer:3,First3,Name3,2022-10-01,2022-10-31,2022-10-31,no,yes,1.00,0.00,Not paid\r\n'
        'F%02d-%s-0000002,payer:2,First2,Name2,2022-10-01,2022-10-31,2022-10-31,no,yes,1.00,1.00,Paid\r\n'
        'F%02d-%s-0000001,payer:1,First1,Name1,2022-10-01,2022-10-31,2022-10-31,yes,no,6.20,1.00,Partially paid\r\n'
    ) % (
        regie.pk,
        invoice3.created_at.strftime('%y-%m'),
        regie.pk,
        invoice2.created_at.strftime('%y-%m'),
        regie.pk,
        invoice1.created_at.strftime('%y-%m'),
    )

    resp = app.get('/manage/invoicing/regie/%s/invoices/?csv&full' % regie.pk)
    assert resp.headers['Content-Type'] == 'text/csv'
    assert resp.headers['Content-Disposition'] == 'attachment; filename="invoices-full.csv"'
    assert len([a for a in resp.text.split('\r\n') if a]) == 1 + 5
    assert resp.text == (
        'Number,Payer ID,Payer first name,Payer last name,Publication date,Payment deadline,Due date,Demat,Direct debit,'
        'Description,Unit amount,Quantity,Total due,Payment type,Paid amount,Status\r\n'
        'F%02d-%s-0000003,payer:3,First3,Name3,2022-10-01,2022-10-31,2022-10-31,no,yes,Event A,1.00,1.00,1.00,,0.00,Not paid\r\n'
        'F%02d-%s-0000002,payer:2,First2,Name2,2022-10-01,2022-10-31,2022-10-31,no,yes,Event AA,1.00,1.00,1.00,"CESU, Cash",1.00,Paid\r\n'
        'F%02d-%s-0000001,payer:1,First1,Name1,2022-10-01,2022-10-31,2022-10-31,yes,no,Event A,1.00,1.20,1.20,Cash,1.00,Partially paid\r\n'
        'F%02d-%s-0000001,payer:1,First1,Name1,2022-10-01,2022-10-31,2022-10-31,yes,no,Event B,2.00,1.00,2.00,,0.00,Not paid\r\n'
        'F%02d-%s-0000001,payer:1,First1,Name1,2022-10-01,2022-10-31,2022-10-31,yes,no,Event A,3.00,1.00,3.00,,0.00,Not paid\r\n'
    ) % (
        regie.pk,
        invoice3.created_at.strftime('%y-%m'),
        regie.pk,
        invoice2.created_at.strftime('%y-%m'),
        regie.pk,
        invoice1.created_at.strftime('%y-%m'),
        regie.pk,
        invoice1.created_at.strftime('%y-%m'),
        regie.pk,
        invoice1.created_at.strftime('%y-%m'),
    )

    # test filters
    params = [
        ({'number': invoice1.formatted_number}, 1),
        ({'number': invoice1.created_at.strftime('%y-%m')}, 3),
        ({'created_at_after': today.strftime('%Y-%m-%d')}, 3),
        ({'created_at_after': tomorrow.strftime('%Y-%m-%d')}, 0),
        ({'created_at_before': yesterday.strftime('%Y-%m-%d')}, 0),
        ({'created_at_before': today.strftime('%Y-%m-%d')}, 3),
        ({'date_payment_deadline_after': '2022-10-31'}, 3),
        ({'date_payment_deadline_after': '2022-11-01'}, 0),
        ({'date_payment_deadline_before': '2022-10-30'}, 0),
        ({'date_payment_deadline_before': '2022-10-31'}, 3),
        ({'date_due_after': '2022-10-31'}, 3),
        ({'date_due_after': '2022-11-01'}, 0),
        ({'date_due_before': '2022-10-30'}, 0),
        ({'date_due_before': '2022-10-31'}, 3),
        ({'payment_number': payment1.formatted_number}, 1),
        ({'payment_number': payment1.created_at.strftime('%y-%m')}, 2),
        ({'payer_external_id': 'payer:1'}, 1),
        ({'payer_external_id': 'payer:2'}, 1),
        ({'payer_first_name': 'first'}, 3),
        ({'payer_first_name': 'first1'}, 1),
        ({'payer_last_name': 'name'}, 3),
        ({'payer_last_name': 'name1'}, 1),
        ({'payer_demat': True}, 1),
        ({'payer_direct_debit': True}, 2),
        ({'user_external_id': 'user:1'}, 3),
        ({'user_external_id': 'user:2'}, 1),
        ({'user_first_name': 'user'}, 3),
        ({'user_first_name': 'user2'}, 1),
        ({'user_last_name': 'name'}, 3),
        ({'user_last_name': 'name1'}, 3),
        (
            {
                'total_amount_min': '1',
                'total_amount_min_lookup': 'gt',
            },
            1,
        ),
        (
            {
                'total_amount_min': '1',
                'total_amount_min_lookup': 'gte',
            },
            3,
        ),
        (
            {
                'total_amount_max': '6.2',
                'total_amount_max_lookup': 'lt',
            },
            2,
        ),
        (
            {
                'total_amount_max': '6.2',
                'total_amount_max_lookup': 'lte',
            },
            3,
        ),
        ({'paid': 'yes'}, 1),
        ({'paid': 'partially'}, 1),
        ({'paid': 'no'}, 1),
        ({'agenda': 'agenda-a'}, 2),
        ({'agenda': 'agenda-b'}, 1),
        ({'event': 'agenda-a@event-a'}, 1),
        ({'event': 'agenda-a@event-aa'}, 1),
        ({'event': 'agenda-b@event-b'}, 1),
    ]
    for param, result in params:
        resp = app.get(
            '/manage/invoicing/regie/%s/invoices/' % regie.pk,
            params=param,
        )
        assert len(resp.pyquery('tr.invoice')) == result
        param['csv'] = True
        resp = app.get(
            '/manage/invoicing/regie/%s/invoices/' % regie.pk,
            params=param,
        )
        assert len([a for a in resp.text.split('\r\n') if a]) == 1 + result

    app.get(
        '/manage/invoicing/ajax/regie/%s/invoice/%s/lines/' % (regie.pk, non_finalized_invoice.pk), status=404
    )


def test_regie_invoice_pdf(app, admin_user):
    regie = Regie.objects.create(label='Foo', invoice_main_colour='#9141ac')
    campaign = Campaign.objects.create(
        regie=regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
        finalized=True,
    )
    assert campaign.invoice_model == 'middle'
    pool = Pool.objects.create(
        campaign=campaign,
        draft=False,
        status='completed',
    )
    invoice = Invoice.objects.create(
        label='Invoice from 01/09/2022 to 30/09/2022',
        date_publication=campaign.date_publication,
        date_payment_deadline=campaign.date_payment_deadline,
        date_due=campaign.date_due,
        regie=regie,
        pool=pool,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=True,
        payer_direct_debit=False,
    )
    invoice.set_number()
    invoice.save()

    InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice,
        quantity=1.2,
        unit_amount=1,
        pool=pool,
        label='Label 11',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        details={
            'agenda': 'agenda-1',
            'primary_event': 'event-1',
            'status': 'presence',
            'check_type': 'foo',
            'check_type_group': 'foobar',
            'check_type_label': 'Foo!',
            'dates': ['2022-09-01', '2022-09-02', '2022-09-03'],
        },
        event_slug='agenda-1@event-1',
        agenda_slug='agenda-1',
        activity_label='Agenda 1',
        description='Thu01, Fri02, Sat03',
    )
    InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 2),
        invoice=invoice,
        quantity=1,
        unit_amount=2,
        pool=pool,
        label='Label 12',
        user_external_id='user:2',
        user_first_name='User2',
        user_last_name='Name2',
    )
    InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 3),
        invoice=invoice,
        quantity=1,
        unit_amount=3,
        pool=pool,
        label='Label 13',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        description='@overtaking@',
    )
    InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice,
        quantity=15,
        unit_amount=1,
        pool=pool,
        label='Agenda foobar',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        details={
            'agenda': 'agenda-2',
            'primary_event': 'event-1',
            'status': 'presence',
            'partial_bookings': True,
        },
        event_slug='agenda-2@event-1',
        agenda_slug='agenda-2',
        activity_label='Agenda 2',
        description='a description',
    )
    InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice,
        quantity=0,
        unit_amount=1,
        pool=pool,
        label='Absence',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        details={
            'agenda': 'agenda-2',
            'primary_event': 'event-1',
            'status': 'absence',
            'partial_bookings': True,
        },
        event_slug='agenda-2@event-1',
        agenda_slug='agenda-2',
        activity_label='Agenda 2',
        description='a description',
    )

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (regie.pk, invoice.pk))
    assert 'color: #9141ac;' in resp
    assert resp.pyquery('#document-label').text() == 'Invoice from 01/09/2022 to 30/09/2022'
    assert resp.pyquery('#regie-label').text() == 'Foo'
    assert resp.pyquery('address#to').text() == 'First1 Name1\n41 rue des kangourous\n99999 Kangourou Ville'
    assert resp.pyquery('dl#informations').text() == 'Invoice number:\nF%02d-%s-0000001\nDate:\n%s' % (
        regie.pk,
        invoice.created_at.strftime('%y-%m'),
        date_format(localtime(invoice.created_at), 'DATE_FORMAT'),
    )
    assert [PyQuery(tr).text() for tr in resp.pyquery('table#lines tr')] == [
        'User1 Name1',
        'Services\nDetails\nUA\nQTY\nTA',
        'Label 11\nAgenda 1\nFoo!\nThu01, Fri02, Sat03\n1.00€\n1.2\n1.20€',
        'Agenda foobar\na description\n1.00€\n15\n15.00€',
        'Absence\na description\n1.00€\n0\n0.00€',
        'Label 13\n\n3.00€\n1\n3.00€',
        'User2 Name2',
        'Services\nDetails\nUA\nQTY\nTA',
        'Label 12\n\n2.00€\n1\n2.00€',
        'Total amount to be paid before: Oct. 31, 2022\n21.20€',
    ]
    assert len(resp.pyquery('table#lines-details')) == 0

    campaign.invoice_model = 'basic'
    campaign.save()
    resp = app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (regie.pk, invoice.pk))
    assert [PyQuery(tr).text() for tr in resp.pyquery('table#lines tr')] == [
        'User1 Name1',
        'Services\nUA\nQTY\nTA',
        'Label 11\nAgenda 1\nFoo!\n1.00€\n1.2\n1.20€',
        'Agenda foobar\n1.00€\n15\n15.00€',
        'Absence\n1.00€\n0\n0.00€',
        'Label 13\n\n3.00€\n1\n3.00€',
        'User2 Name2',
        'Services\nUA\nQTY\nTA',
        'Label 12\n\n2.00€\n1\n2.00€',
        'Total amount to be paid before: Oct. 31, 2022\n21.20€',
    ]
    assert len(resp.pyquery('table#lines-details')) == 0

    campaign.invoice_model = 'full'
    campaign.save()
    resp = app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (regie.pk, invoice.pk))
    assert [PyQuery(tr).text() for tr in resp.pyquery('table#lines tr')] == [
        'User1 Name1',
        'Services\nUA\nQTY\nTA',
        'Label 11\nAgenda 1\nFoo!\n1.00€\n1.2\n1.20€',
        'Agenda foobar\n1.00€\n15\n15.00€',
        'Absence\n1.00€\n0\n0.00€',
        'Label 13\n\n3.00€\n1\n3.00€',
        'User2 Name2',
        'Services\nUA\nQTY\nTA',
        'Label 12\n\n2.00€\n1\n2.00€',
        'Total amount to be paid before: Oct. 31, 2022\n21.20€',
    ]
    assert len(resp.pyquery('table#lines-details')) == 1
    assert [PyQuery(tr).text() for tr in resp.pyquery('table#lines-details tr')] == [
        'User1 Name1',
        'Services\nDetails',
        'Label 11\nAgenda 1\nFoo!\nThu01, Fri02, Sat03',
        'Agenda foobar\na description',
        'Absence\na description',
    ]

    app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (0, invoice.pk), status=404)
    app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (regie.pk, 0), status=404)
    other_regie = Regie.objects.create(label='Foo')
    app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (other_regie.pk, invoice.pk), status=404)

    appearance_settings = AppearanceSettings.singleton()
    appearance_settings.address = '<p>Foo bar<br>Streetname</p>'
    appearance_settings.extra_info = '<p>Opening hours...</p>'
    appearance_settings.save()
    resp = app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (regie.pk, invoice.pk))
    assert appearance_settings.address in resp.text
    assert appearance_settings.extra_info in resp.text

    campaign.finalized = False
    campaign.save()
    app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (regie.pk, invoice.pk), status=404)
    invoice.pool = None
    invoice.save()
    app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (regie.pk, invoice.pk), status=200)
    invoice.cancelled_at = now()
    invoice.save()
    app.get('/manage/invoicing/regie/%s/invoice/%s/pdf/?html' % (regie.pk, invoice.pk), status=404)


def test_regie_invoice_payments_pdf(app, admin_user):
    regie = Regie.objects.create(
        label='Foo',
        cashier_name='Le régisseur principal',
        city_name='Kangourou Ville',
        invoice_main_colour='#9141ac',
    )
    campaign = Campaign.objects.create(
        regie=regie,
        date_start=datetime.date(2022, 9, 1),
        date_end=datetime.date(2022, 10, 1),
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
        finalized=True,
    )
    pool = Pool.objects.create(
        campaign=campaign,
        draft=False,
        status='completed',
    )
    PaymentType.create_defaults(regie)
    invoice = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=True,
        payer_direct_debit=False,
        label='Invoice Label',
    )
    invoice.set_number()
    invoice.save()

    line = InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice,
        quantity=1,
        unit_amount=40,
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )

    payment1 = Payment.objects.create(
        regie=regie,
        amount=5,
        payment_type=PaymentType.objects.get(regie=regie, slug='check'),
    )
    payment1.set_number()
    payment1.save()
    InvoiceLinePayment.objects.create(
        payment=payment1,
        line=line,
        amount=5,
    )
    payment2 = Payment.objects.create(
        regie=regie,
        amount=35,
        payment_type=PaymentType.objects.get(regie=regie, slug='online'),
    )
    payment2.set_number()
    payment2.save()
    InvoiceLinePayment.objects.create(
        payment=payment2,
        line=line,
        amount=35,
    )
    invoice.refresh_from_db()
    assert invoice.remaining_amount == 0

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/invoice/%s/payments/pdf/?html' % (regie.pk, invoice.pk))
    assert 'color: #9141ac;' in resp
    assert resp.pyquery('#document-label').text() == 'Payments certificate'
    assert resp.pyquery('.address-to-container').text() == (
        'Invoiced account:\nFirst1 Name1 (1)\nInvoicing address:\n41 rue des kangourous\n99999 Kangourou Ville\n'
        'Invoice number:\nF%02d-%s-0000001\nInvoice object:\nInvoice Label'
    ) % (
        regie.pk,
        invoice.created_at.strftime('%y-%m'),
    )
    assert (
        resp.pyquery('p#informations').text()
        == 'Hereby certifies that I have received the amount of 40.00€ for account 1 First1 Name1 as follows:'
    )
    assert [PyQuery(tr).text() for tr in resp.pyquery.find('thead tr')] == [
        'Number\nDate\nPayment type\nAmount',
    ]
    assert [PyQuery(tr).text() for tr in resp.pyquery.find('tbody tr')] == [
        'R%02d-%s-0000001\n%s\nCheck\n5.00€'
        % (
            regie.pk,
            payment1.created_at.strftime('%y-%m'),
            payment1.created_at.strftime('%d/%m/%Y'),
        ),
        'R%02d-%s-0000002\n%s\nOnline\n35.00€'
        % (
            regie.pk,
            payment2.created_at.strftime('%y-%m'),
            payment2.created_at.strftime('%d/%m/%Y'),
        ),
    ]
    assert resp.pyquery(
        '#regie-signature'
    ).text() == 'Le régisseur principal\nKangourou Ville, on %s' % invoice.created_at.strftime('%d/%m/%Y')

    app.get('/manage/invoicing/regie/%s/invoice/%s/payments/pdf/?html' % (0, invoice.pk), status=404)
    app.get('/manage/invoicing/regie/%s/invoice/%s/payments/pdf/?html' % (regie.pk, 0), status=404)
    other_regie = Regie.objects.create(label='Foo')
    app.get(
        '/manage/invoicing/regie/%s/invoice/%s/payments/pdf/?html' % (other_regie.pk, invoice.pk), status=404
    )
    line = InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice,
        quantity=1,
        unit_amount=40,
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    invoice.refresh_from_db()
    assert invoice.remaining_amount > 0
    app.get('/manage/invoicing/regie/%s/invoice/%s/payments/pdf/?html' % (regie.pk, invoice.pk), status=404)

    line.delete()
    invoice.refresh_from_db()
    assert invoice.remaining_amount == 0
    invoice.pool = pool
    invoice.save()
    app.get('/manage/invoicing/regie/%s/invoice/%s/payments/pdf/?html' % (regie.pk, invoice.pk), status=200)
    campaign.finalized = False
    campaign.save()
    app.get('/manage/invoicing/regie/%s/invoice/%s/payments/pdf/?html' % (regie.pk, invoice.pk), status=404)
    campaign.finalized = True
    campaign.save()
    invoice.cancelled_at = now()
    invoice.save()
    app.get('/manage/invoicing/regie/%s/invoice/%s/payments/pdf/?html' % (regie.pk, invoice.pk), status=404)
