import datetime

import pytest
from django.utils.timezone import now
from pyquery import PyQuery

from lingo.agendas.models import Agenda
from lingo.invoicing.models import (
    Invoice,
    InvoiceLine,
    InvoiceLinePayment,
    Payment,
    PaymentCancellationReason,
    PaymentDocket,
    PaymentType,
    Regie,
)
from tests.utils import login

pytestmark = pytest.mark.django_db


def test_regie_payments(app, admin_user):
    regie = Regie.objects.create(label='Foo')
    Agenda.objects.create(label='Agenda A', regie=regie)
    Agenda.objects.create(label='Agenda B', regie=regie)
    PaymentType.create_defaults(regie)
    invoice1 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=True,
        payer_direct_debit=False,
    )
    invoice1.set_number()
    invoice1.save()
    invoice2 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=False,
        payer_direct_debit=True,
    )
    invoice2.set_number()
    invoice2.save()
    invoice3 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
        regie=regie,
        payer_external_id='payer:3',
        payer_first_name='First3',
        payer_last_name='Name3',
        payer_address='43 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=False,
        payer_direct_debit=True,
    )
    invoice3.set_number()
    invoice3.save()

    invoice_line1 = InvoiceLine.objects.create(
        slug='event-a-foo-bar',
        label='Event A',
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice1,
        quantity=1,
        unit_amount=40,
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
        details={
            'agenda': 'agenda-a',
            'primary_event': 'event-a',
        },
        event_slug='agenda-a@event-a',
        agenda_slug='agenda-a',
        activity_label='Agenda A',
    )
    invoice_line2 = InvoiceLine.objects.create(
        slug='agenda-b@event-b',  # non recurring event
        label='Event B',
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice2,
        quantity=1,
        unit_amount=50,
        event_slug='agenda-b@event-b',
        agenda_slug='agenda-b',
        activity_label='Agenda B',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    invoice_line3 = InvoiceLine.objects.create(
        slug='injected',
        label='Event A',
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice3,
        quantity=1,
        unit_amount=60,
        event_slug='injected',
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )

    payment1 = Payment.objects.create(
        regie=regie,
        amount=35,
        payment_type=PaymentType.objects.get(regie=regie, slug='cash'),
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payment_info={
            'check_number': '123456',
        },
    )
    payment1.set_number()
    payment1.save()
    InvoiceLinePayment.objects.create(
        payment=payment1,
        line=invoice_line1,
        amount=35,
    )

    payment2 = Payment.objects.create(
        regie=regie,
        amount=55,
        payment_type=PaymentType.objects.get(regie=regie, slug='check'),
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payment_info={
            'check_number': '123456',
            'check_issuer': 'Foo',
            'check_bank': 'Bar',
            'payment_reference': 'Ref',
        },
    )
    payment2.set_number()
    payment2.save()
    InvoiceLinePayment.objects.create(
        payment=payment2,
        line=invoice_line1,
        amount=5,
    )
    InvoiceLinePayment.objects.create(
        payment=payment2,
        line=invoice_line2,
        amount=50,
    )

    docket = PaymentDocket.objects.create(
        regie=regie, date_end=now().date() + datetime.timedelta(days=1), draft=False
    )
    docket.set_number()
    docket.save()
    payment3 = Payment.objects.create(
        regie=regie,
        amount=2,
        payment_type=PaymentType.objects.get(regie=regie, slug='creditcard'),
        payer_external_id='payer:3',
        payer_first_name='First3',
        payer_last_name='Name3',
        payer_address='43 rue des kangourous\n99999 Kangourou Ville',
        docket=docket,
    )
    payment3.set_number()
    payment3.save()
    InvoiceLinePayment.objects.create(
        payment=payment3,
        line=invoice_line3,
        amount=2,
    )

    payment4 = Payment.objects.create(
        regie=regie,
        amount=2,
        payment_type=PaymentType.objects.get(regie=regie, slug='check'),
        payer_external_id='payer:3',
        payer_first_name='First3',
        payer_last_name='Name3',
        payer_address='43 rue des kangourous\n99999 Kangourou Ville',
        cancelled_at=now(),
        cancelled_by=admin_user,
        cancellation_reason=PaymentCancellationReason.objects.create(label='Uncovered check'),
        cancellation_description='foo bar\nblah',
    )
    payment4.set_number()
    payment4.save()

    invoice1.refresh_from_db()
    assert invoice1.remaining_amount == 0
    assert invoice1.paid_amount == 40
    invoice2.refresh_from_db()
    assert invoice2.remaining_amount == 0
    assert invoice2.paid_amount == 50
    invoice3.refresh_from_db()
    assert invoice3.remaining_amount == 58
    assert invoice3.paid_amount == 2

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/payments/' % regie.pk)
    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % payment1.pk
    ).text() == 'Payment R%02d-%s-0000001 dated %s from First1 Name1, amount 35.00€ (Cash) - download' % (
        regie.pk,
        payment1.created_at.strftime('%y-%m'),
        payment1.created_at.strftime('%d/%m/%Y'),
    )
    assert [
        PyQuery(tr).text() for tr in resp.pyquery('tr[data-related-invoicing-element-id="%s"]' % payment1.pk)
    ] == [
        'Invoice\nAmount charged\nAmount assigned',
        'F%02d-%s-0000001\n40.00€\n35.00€'
        % (
            regie.pk,
            invoice1.created_at.strftime('%y-%m'),
        ),
        'Number: 123456',
        'Cancel payment',
    ]
    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % payment2.pk
    ).text() == 'Payment R%02d-%s-0000002 dated %s from First1 Name1, amount 55.00€ (Check) - download' % (
        regie.pk,
        payment2.created_at.strftime('%y-%m'),
        payment2.created_at.strftime('%d/%m/%Y'),
    )
    assert [
        PyQuery(tr).text() for tr in resp.pyquery('tr[data-related-invoicing-element-id="%s"]' % payment2.pk)
    ] == [
        'Invoice\nAmount charged\nAmount assigned',
        'F%02d-%s-0000001\n40.00€\n5.00€'
        % (
            regie.pk,
            invoice1.created_at.strftime('%y-%m'),
        ),
        'F%02d-%s-0000002\n50.00€\n50.00€'
        % (
            regie.pk,
            invoice2.created_at.strftime('%y-%m'),
        ),
        'Issuer: Foo',
        'Bank/Organism: Bar',
        'Number: 123456',
        'Reference: Ref',
        'Cancel payment',
    ]
    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % payment3.pk
    ).text() == 'Payment R%02d-%s-0000003 dated %s from First3 Name3, amount 2.00€ (Credit card) - download' % (
        regie.pk,
        payment3.created_at.strftime('%y-%m'),
        payment3.created_at.strftime('%d/%m/%Y'),
    )
    assert [
        PyQuery(tr).text() for tr in resp.pyquery('tr[data-related-invoicing-element-id="%s"]' % payment3.pk)
    ] == [
        'Invoice\nAmount charged\nAmount assigned',
        'F%02d-%s-0000003\n60.00€\n2.00€'
        % (
            regie.pk,
            invoice3.created_at.strftime('%y-%m'),
        ),
        'Docket: B%02d-%s-0000001' % (regie.pk, docket.created_at.strftime('%y-%m')),
        'Cancel payment',
    ]
    assert resp.pyquery(
        'tr[data-invoicing-element-id="%s"]' % payment4.pk
    ).text() == 'Cancelled Payment R%02d-%s-0000004 dated %s from First3 Name3, amount 2.00€ (Check) - download' % (
        regie.pk,
        payment4.created_at.strftime('%y-%m'),
        payment4.created_at.strftime('%d/%m/%Y'),
    )
    assert [
        PyQuery(tr).text() for tr in resp.pyquery('tr[data-related-invoicing-element-id="%s"]' % payment4.pk)
    ] == [
        'Invoice\nAmount charged\nAmount assigned',
        'No assignments for this payment',
        'Cancelled on: %s' % payment4.cancelled_at.strftime('%d/%m/%Y %H:%M'),
        'Cancelled by: admin',
        'Reason: Uncovered check',
        'Description: foo bar\nblah',
    ]

    resp = app.get('/manage/invoicing/regie/%s/payments/?csv' % regie.pk)
    assert resp.headers['Content-Type'] == 'text/csv'
    assert resp.headers['Content-Disposition'] == 'attachment; filename="payments.csv"'
    assert len([a for a in resp.text.split('\r\n') if a]) == 1 + 5
    assert resp.text == (
        'Number,Invoice number,Date,Payer ID,Payer first name,Payer last name,Payment type,Amount assigned,'
        'Total amount,Issuer,Bank/Organism,Number,Reference,Cancelled on,Cancellation reason\r\n'
        'R%02d-%s-0000004,,%s,payer:3,First3,Name3,Check,,2.00,,,,,%s,Uncovered check\r\n'
        'R%02d-%s-0000003,F%02d-%s-0000003,%s,payer:3,First3,Name3,Credit card,2.00,2.00,,,,,,\r\n'
        'R%02d-%s-0000002,F%02d-%s-0000001,%s,payer:1,First1,Name1,Check,5.00,55.00,Foo,Bar,123456,Ref,,\r\n'
        'R%02d-%s-0000002,F%02d-%s-0000002,%s,payer:1,First1,Name1,Check,50.00,55.00,Foo,Bar,123456,Ref,,\r\n'
        'R%02d-%s-0000001,F%02d-%s-0000001,%s,payer:1,First1,Name1,Cash,35.00,35.00,,,123456,,,\r\n'
    ) % (
        regie.pk,
        payment4.created_at.strftime('%y-%m'),
        payment4.created_at.strftime('%Y-%m-%d'),
        payment4.cancelled_at.isoformat(),
        regie.pk,
        payment3.created_at.strftime('%y-%m'),
        regie.pk,
        invoice3.created_at.strftime('%y-%m'),
        payment3.created_at.strftime('%Y-%m-%d'),
        regie.pk,
        payment2.created_at.strftime('%y-%m'),
        regie.pk,
        invoice1.created_at.strftime('%y-%m'),
        payment2.created_at.strftime('%Y-%m-%d'),
        regie.pk,
        payment2.created_at.strftime('%y-%m'),
        regie.pk,
        invoice2.created_at.strftime('%y-%m'),
        payment2.created_at.strftime('%Y-%m-%d'),
        regie.pk,
        payment1.created_at.strftime('%y-%m'),
        regie.pk,
        invoice1.created_at.strftime('%y-%m'),
        payment1.created_at.strftime('%Y-%m-%d'),
    )

    # test filters
    today = datetime.date.today()
    params = [
        ({'number': payment1.formatted_number}, 1, 1),
        ({'number': payment1.created_at.strftime('%y-%m')}, 4, 5),
        ({'created_at_after': today.strftime('%Y-%m-%d')}, 4, 5),
        ({'created_at_after': (today + datetime.timedelta(days=1)).strftime('%Y-%m-%d')}, 0, 0),
        ({'created_at_before': (today - datetime.timedelta(days=1)).strftime('%Y-%m-%d')}, 0, 0),
        ({'created_at_before': today.strftime('%Y-%m-%d')}, 4, 5),
        ({'invoice_number': invoice1.formatted_number}, 2, 3),
        ({'invoice_number': invoice1.created_at.strftime('%y-%m')}, 3, 4),
        ({'payer_external_id': 'payer:1'}, 2, 3),
        ({'payer_external_id': 'payer:3'}, 2, 2),
        ({'payer_first_name': 'first'}, 4, 5),
        ({'payer_first_name': 'first1'}, 2, 3),
        ({'payer_last_name': 'name'}, 4, 5),
        ({'payer_last_name': 'name1'}, 2, 3),
        ({'payment_type': PaymentType.objects.get(slug='cash').pk}, 1, 1),
        ({'payment_type': PaymentType.objects.get(slug='check').pk}, 2, 3),
        (
            {
                'amount_min': '2',
                'amount_min_lookup': 'gt',
            },
            2,
            3,
        ),
        (
            {
                'amount_min': '2',
                'amount_min_lookup': 'gte',
            },
            4,
            5,
        ),
        (
            {
                'amount_max': '55',
                'amount_max_lookup': 'lt',
            },
            3,
            3,
        ),
        (
            {
                'amount_max': '55',
                'amount_max_lookup': 'lte',
            },
            4,
            5,
        ),
        ({'agenda': 'agenda-a'}, 2, 3),
        ({'agenda': 'agenda-b'}, 1, 2),
        ({'event': 'agenda-a@event-a'}, 2, 3),
        ({'event': 'agenda-b@event-b'}, 1, 2),
        ({'cancelled': 'yes'}, 1, 1),
        ({'cancelled': 'no'}, 3, 4),
    ]
    for param, result, csv_result in params:
        resp = app.get(
            '/manage/invoicing/regie/%s/payments/' % regie.pk,
            params=param,
        )
        assert len(resp.pyquery('tr.payment')) == result
        param['csv'] = True
        resp = app.get(
            '/manage/invoicing/regie/%s/payments/' % regie.pk,
            params=param,
        )
        assert len([a for a in resp.text.split('\r\n') if a]) == 1 + csv_result


def test_regie_payment_pdf(app, admin_user):
    regie = Regie.objects.create(
        label='Foo',
        cashier_name='Le régisseur principal',
        city_name='Kangourou Ville',
        invoice_main_colour='#9141ac',
    )
    PaymentType.create_defaults(regie)
    invoice1 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=True,
        payer_direct_debit=False,
    )
    invoice1.set_number()
    invoice1.save()
    invoice2 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=False,
        payer_direct_debit=True,
    )
    invoice2.set_number()
    invoice2.save()

    invoice_line1 = InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice1,
        quantity=1,
        unit_amount=40,
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    invoice_line2 = InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice2,
        quantity=1,
        unit_amount=50,
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )

    payment = Payment.objects.create(
        regie=regie,
        amount=55,
        payment_type=PaymentType.objects.get(regie=regie, slug='check'),
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
    )
    payment.set_number()
    payment.save()
    InvoiceLinePayment.objects.create(
        payment=payment,
        line=invoice_line1,
        amount=5,
    )
    InvoiceLinePayment.objects.create(
        payment=payment,
        line=invoice_line2,
        amount=50,
    )

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/payment/%s/pdf/?html' % (regie.pk, payment.pk))
    assert 'color: #9141ac;' in resp
    assert resp.pyquery('#document-label').text() == 'Payment receipt'
    assert resp.pyquery('.address-to-container').text() == (
        'Invoiced account:\nFirst1 Name1 (1)\nInvoicing address:\n41 rue des kangourous\n99999 Kangourou Ville'
    )
    assert resp.pyquery('p#informations').text() == (
        'Hereby certifies that I have received a payment of type CHECK in the amount of 55.00€ (number R%02d-%s-0000001), for account 1 First1 Name1:'
    ) % (
        regie.pk,
        payment.created_at.strftime('%y-%m'),
    )
    assert [PyQuery(tr).text() for tr in resp.pyquery.find('thead tr')] == [
        'Invoice number\nInvoice object\nAmount charged\nAmount assigned'
    ]
    assert [PyQuery(tr).text() for tr in resp.pyquery.find('tbody tr')] == [
        'F%02d-%s-0000001\n40.00€\n5.00€'
        % (
            regie.pk,
            invoice1.created_at.strftime('%y-%m'),
        ),
        'F%02d-%s-0000002\n50.00€\n50.00€'
        % (
            regie.pk,
            invoice2.created_at.strftime('%y-%m'),
        ),
    ]
    assert resp.pyquery(
        '#regie-signature'
    ).text() == 'Le régisseur principal\nKangourou Ville, on %s' % payment.created_at.strftime('%d/%m/%Y')

    resp = app.get('/manage/invoicing/regie/%s//payment/%s/pdf/?html' % (0, payment.pk), status=404)
    resp = app.get('/manage/invoicing/regie/%s//payment/%s/pdf/?html' % (regie.pk, 0), status=404)
    other_regie = Regie.objects.create(label='Foo')
    resp = app.get(
        '/manage/invoicing/regie/%s/payment/%s/pdf/?html' % (other_regie.pk, payment.pk), status=404
    )


def test_regie_payment_cancel(app, admin_user):
    regie = Regie.objects.create(
        label='Foo',
    )
    docket = PaymentDocket.objects.create(
        regie=regie, date_end=now().date() + datetime.timedelta(days=1), draft=False
    )
    docket.set_number()
    docket.save()
    PaymentType.create_defaults(regie)
    invoice1 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=True,
        payer_direct_debit=False,
    )
    invoice1.set_number()
    invoice1.save()
    invoice2 = Invoice.objects.create(
        date_publication=datetime.date(2022, 10, 1),
        date_payment_deadline=datetime.date(2022, 10, 31),
        date_due=datetime.date(2022, 10, 31),
        date_debit=datetime.date(2022, 11, 15),
        regie=regie,
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        payer_demat=False,
        payer_direct_debit=True,
    )
    invoice2.set_number()
    invoice2.save()

    invoice_line1 = InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice1,
        quantity=1,
        unit_amount=40,
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )
    invoice_line2 = InvoiceLine.objects.create(
        event_date=datetime.date(2022, 9, 1),
        invoice=invoice2,
        quantity=1,
        unit_amount=50,
        user_external_id='user:1',
        user_first_name='User1',
        user_last_name='Name1',
    )

    payment = Payment.objects.create(
        regie=regie,
        amount=55,
        payment_type=PaymentType.objects.get(regie=regie, slug='check'),
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
    )
    payment.set_number()
    payment.save()
    InvoiceLinePayment.objects.create(
        payment=payment,
        line=invoice_line1,
        amount=5,
    )
    InvoiceLinePayment.objects.create(
        payment=payment,
        line=invoice_line2,
        amount=50,
    )
    cancellation_reason = PaymentCancellationReason.objects.create(label='Uncovered check')
    PaymentCancellationReason.objects.create(label='Disabled', disabled=True)
    payment2 = Payment.objects.create(
        regie=regie,
        amount=5,
        payment_type=PaymentType.objects.get(regie=regie, slug='check'),
        payer_external_id='payer:1',
        payer_first_name='First1',
        payer_last_name='Name1',
        payer_address='41 rue des kangourous\n99999 Kangourou Ville',
        docket=docket,
    )
    payment2.set_number()
    payment2.save()
    InvoiceLinePayment.objects.create(
        payment=payment2,
        line=invoice_line1,
        amount=5,
    )

    app = login(app)
    resp = app.get('/manage/invoicing/regie/%s/payments/' % regie.pk)
    resp = resp.click(href='/manage/invoicing/regie/%s/payment/%s/cancel/' % (regie.pk, payment.pk))
    assert 'This payment is included in a docket.' not in resp
    assert 'If you cancel this payment, the docket will be modified.' not in resp
    assert resp.form['cancellation_reason'].options == [
        ('', True, '---------'),
        (str(cancellation_reason.pk), False, 'Uncovered check'),
    ]
    resp.form['cancellation_reason'] = cancellation_reason.pk
    resp.form['cancellation_description'] = 'foo bar blah'
    resp = resp.form.submit()
    assert resp.location.endswith(
        '/manage/invoicing/regie/%s/payments/?number=%s' % (regie.pk, payment.formatted_number)
    )
    payment.refresh_from_db()
    assert payment.cancelled_at is not None
    assert payment.cancelled_by == admin_user
    assert payment.cancellation_reason == cancellation_reason
    assert payment.cancellation_description == 'foo bar blah'
    assert payment.invoicelinepayment_set.count() == 0
    assert payment2.invoicelinepayment_set.count() == 1

    # already cancelled
    app.get('/manage/invoicing/regie/%s/payment/%s/cancel/' % (regie.pk, payment.pk), status=404)
    payment.cancelled_at = None
    payment.save()
    other_regie = Regie.objects.create(label='Foo')
    app.get('/manage/invoicing/regie/%s/payment/%s/cancel/' % (other_regie.pk, payment.pk), status=404)

    # payment in docket
    resp = app.get('/manage/invoicing/regie/%s/payment/%s/cancel/' % (regie.pk, payment2.pk))
    assert 'This payment is included in a docket.' in resp
    assert 'If you cancel this payment, the docket will be modified.' in resp
    resp.form['cancellation_reason'] = cancellation_reason.pk
    resp.form['cancellation_description'] = 'foo bar blah'
    resp = resp.form.submit()
    assert resp.location.endswith(
        '/manage/invoicing/regie/%s/payments/?number=%s' % (regie.pk, payment2.formatted_number)
    )
    payment2.refresh_from_db()
    assert payment2.cancelled_at is not None
    assert payment2.cancelled_by == admin_user
    assert payment2.cancellation_reason == cancellation_reason
    assert payment2.cancellation_description == 'foo bar blah'
    assert payment2.invoicelinepayment_set.count() == 0
